import { apiDomain, auth as authPaths } from '../apiPaths'
import apiRequest from '../request'
import { MODELS, actionValues, roles, ACTIONS } from '../../constants'

apiRequest.interceptors.request.use(
  config => {
    const token = localStorage.getItem('accessToken')

    config.headers = {
      'Authorization': `Bearer ${token}`,
      'Accept': 'application/json',
      'Content-Type': 'application/json',
      ...config.headers
    }
    config.baseURL = config.baseURL || apiDomain
    return config
  },
  async (error) => Promise.reject(error)
)

apiRequest.interceptors.response.use(
  response => formatApiResponse(null, response),
  async (error) => {
    const originalRequest = error.config
    if (
      originalRequest.url !== authPaths.refreshToken() &&
      error.response &&
      error.response.status === 401 &&
      !originalRequest.retry
    ) {
      originalRequest.retry = true
      const accessToken = await refreshAccessToken(originalRequest)
      if (!accessToken) {
        return formatApiResponse(error)
      }
      apiRequest.defaults.headers.common['Authorization'] = 'Bearer ' + accessToken
      return apiRequest(originalRequest)
    }
    return formatApiResponse(error)
  }
)

const formatApiResponse = (err, data) => {
  let response = {
    isError: false,
    errorMessage: false,
    statusCode: 0,
    errorCode: 0,
    data: null
  }
  if (typeof (err || data).isError == 'boolean') {
    return err || data
  }

  if (err) {
    let errorMessage = err?.response?.data?.message || err.message

    response.isError = true
    response.errorMessage = errorMessage
    response.statusCode = err.statusCode
    response.errorCode = -1
    response.rawError = err
  }
  if (data) {
    response.data = data.data
  }

  return response
}

export const login = async (data) => {
  const result = await apiRequest.post(authPaths.login(), data)
  if (!result.isError) storeUserInfo(result.data)
  return result
}

function storeUserInfo(user) {
  if (user) {
    let { accessToken, refreshToken } = user
    if (accessToken) {
      delete user.accessToken
      localStorage.setItem('accessToken', accessToken)
    }
    if (refreshToken) {
      delete user.refreshToken
      localStorage.setItem('refreshToken', refreshToken)
    }
    localStorage.setItem('userProfile', JSON.stringify(user))
  }
}

export const checkAuth = async () => {
  const response = await apiRequest.get(authPaths.self())

  if (!response.isError) {
    response.data.panels = (response.data.permissions || []).reduce((result, p) => {
      result[p.model] = (p.permission & actionValues[ACTIONS.VIEW]) > 0
      return result
    }, {})
    storeUserInfo(response.data)
  }
  return response
}

export const fetchUserInfo = async () => {
  let userCache = localStorage.getItem('userProfile')
  if (userCache) {
    try {
      return formatApiResponse(null, {data: JSON.parse(userCache)})
    }
    catch (err) {
      console.log('error when parse data')
      console.log(userCache)
    }
  }
  return checkAuth()
}

export const register = async (form) => {
  const result = await apiRequest.postAsForm(authPaths.register(), form)

  return result
}

export const logout = async() => {
  localStorage.removeItem('accessToken')
  localStorage.removeItem('refreshToken')
  localStorage.removeItem('userProfile')
  window.location.href="/";
}

async function refreshAccessToken(originalRequest) {
  const refreshToken = localStorage.getItem('refreshToken')
  const response = await apiRequest.post(authPaths.refreshToken(), { refreshToken })
  if (!response.isError) storeUserInfo(response.data)
  return response.data ? response.data.accessToken : ""
}

export const getSubRoles = (role) => {
  let curRole = roles.find(r => r.value === role)
  if (!curRole) return []
  return roles.filter(role => role.level >= curRole.level)
}

export const checkPermission = async (model, action) => {
  const res = await fetchUserInfo()
  if (!res || res.isError) {
    return false
  }
  const userInfo = res.data
  const permission = userInfo.permissions.find(p => p.model === MODELS[model])

  return !!((permission?.permission & actionValues[action]) > 0)
}