import React, { useEffect, useState } from 'react'
import {
  CSelect,
  CFormGroup,
  CInput,
  CInputGroup,
  CInputGroupAppend,
  CInputGroupText
} from '@coreui/react'
import DateRangePicker from 'react-bootstrap-daterangepicker'

const Search = (props) => {
  const genSearchElem = (field, idx) => {
    switch (field.type) {
      case 'text': {
        return (
          <TextSearch
            name={field.name}
            key={idx}
            search={props.search}
            gridSize={field.gridSize}
            placeholder={field.placeholder}
            autocomplete={field.autocomplete}
            suggestions={field.suggestions}
          />
        )
      }
      case 'select': {
        return (
          <SelectSearch
            name={field.name}
            key={idx}
            search={props.search}
            placeholder={field.placeholder}
            options={field.options}
            gridSize={field.gridSize}
          />
        )
      }
      case 'dateRange': {
        return (
          <DateRangeSearch
            name={field.name}
            key={idx}
            search={props.search}
            placeholder={field.placeholder}
            field={field.field}
            gridSize={field.gridSize}
          />
        )
      }
      default: return <></>
    }
  }
  return (
    <div className={props.className || "row mx-0 justify-content-start align-items-center flex-wrap"}>
      {props.items.map((item, idx) => genSearchElem(item, idx))}
    </div>
  )
}

const SelectSearch = (props) => {
  const onSelect = ({ target: { name, value }}) => {
    props.search(name, value)
  }

  return (
    <CFormGroup className={"mb-4 px-1 " + (props.gridSize ? `col-${props.gridSize}` : "")}>
      <CSelect
        name={props.name}
        onChange={onSelect}
        className="pl-1"
      >
        <option value="">{props.placeholder || "Select"}</option>
        {props.options.map((option, idx) => (
          <option value={option.value} key={idx}>{option.text}</option>
        ))}
      </CSelect>
    </CFormGroup>
  )
}

const TextSearch = (props) => {
  const [textSearch, setTextSearch] = useState("")
  const [changeFilter, setChangeFilter] = useState(false)
  const [focusIndex, setFocusIndex] = useState(-1)
  const [filteredSuggestions, setFilteredSuggesstions] = useState([])
  const onSearch = (e) => {
    if (changeFilter) {
      props.search(props.name, textSearch)
      setChangeFilter(false)
      setFilteredSuggesstions([])
    }
  }
  const onChangeValue = ({ target: { value }}) => {
    setTextSearch(value)
    setChangeFilter(true)
    if (props.autocomplete) {
      filterSuggestion(value)
      setFocusIndex(-1)
    }
  }

  const showAutocomplete = () => {
    if (props.autocomplete) filterSuggestion(textSearch)
  }

  const filterSuggestion = (value) => {
    setFilteredSuggesstions(
      props.suggestions.filter(sug => value && sug.toUpperCase().indexOf(value.toUpperCase()) > -1).slice(0, props.suggestionSize || 5)
    )
  }

  const keyDown = (e) => {
    switch (e.key) {
      case 'ArrowDown': {
        if (!props.autocomplete || !props.suggestions?.length) return
        let idx = focusIndex + 1
        if (idx >= props.suggestions?.length) idx = 0
        return setFocusIndex(idx)
      }
      case 'ArrowUp': {
        if (!props.autocomplete || !props.suggestions?.length) return
        let idx = focusIndex - 1
        if (idx < 0) idx = props.suggestions?.length - 1
        return setFocusIndex(idx)
      }
      case 'Enter': {
        if (props.autocomplete && filteredSuggestions.length && focusIndex > -1) return onChooseSuggestion()
        return onSearch()
      }
      default: return
    }
  }

  const onChooseSuggestion = (idx) => {
    let selectIdx = idx === undefined ? focusIndex : idx
    if (selectIdx < 0) return
    const suggestion = filteredSuggestions[selectIdx]
    if (!suggestion) return
    setFocusIndex(-1)
    setFilteredSuggesstions([])
    setChangeFilter(false)
    setTextSearch(suggestion)
    props.search(props.name, suggestion)
  }

  return (
    <CFormGroup className={"mb-4 px-1 " + (props.gridSize ? `col-${props.gridSize}` : "")}>
      <CInputGroup className={props.autocomplete ? "autocomplete" : ""}>
        <CInput
          onChange={onChangeValue}
          value={textSearch}
          id="name"
          name="name"
          onClick={showAutocomplete}
          placeholder={props.placeholder}
          type="text"
          onKeyDown={keyDown}
          autoComplete="off"
        />
        {!!props.autocomplete && !!filteredSuggestions.length &&
          <div className="autocomplete-items">
              {
                filteredSuggestions.map((suggestion, idx) => (
                  <div className={idx === focusIndex ? "autocomplete-active" : ""} key={idx} onClick={() => onChooseSuggestion(idx)}>
                    {suggestion}
                  </div>
                ))
              }
          </div>
        }
        <CInputGroupAppend>
          <CInputGroupText onClick={onSearch} className="cursor-pointer"><i className="fas fa-search" style={{ width: '10px' }}/></CInputGroupText>
        </CInputGroupAppend>
      </CInputGroup>
    </CFormGroup>
  )
}

const DateRangeSearch = (props) => {
  const inputId = `input-picker=#${props.name}`
  useEffect(() => {
    document.getElementById(inputId).value = ""
  }, [inputId])
  const onApplyDate = (_, picker) => {
    const startDate = picker.startDate
    const endDate = picker.endDate
    props.search(props.name, { startDate: startDate.format('YYYY-MM-DD'), endDate: endDate.format('YYYY-MM-DD') })
  }
  const onClear = () => {
    props.search(props.name, null)
    document.getElementById(inputId).value = ""
  }

  const showPicker = () => {
    document.getElementById(inputId)?.click()
  }

  return (
    <CFormGroup className={"mb-4 px-1 date-range-dropdown " + (props.gridSize ? `col-${props.gridSize}` : "")}>
      <CInputGroup>
        <CInputGroupAppend>
          <CInputGroupText className="cursor-pointer" onClick={showPicker}>
            <i className="fas fa-calendar pr-3" style={{ width: '10px' }}/>
            {props.field}
          </CInputGroupText>
        </CInputGroupAppend>
        <DateRangePicker
          onApply={onApplyDate}
          initialSettings={{
            locale: {
              cancelLabel: "Clear"
            }
          }}
          autoUpdateInput={false}
          onCancel={onClear}
        >
          <input
            id={inputId}
            type="text"
            className="cursor-pointer form-control"
            alt={props.placeholder}
            placeholder={`${props.placeholder}...`}
          />
        </DateRangePicker>
      </CInputGroup>
    </CFormGroup>
  )
}

export default Search