import React, { useState, useEffect } from 'react'
import {
  CCard,
  CCardBody,
  CCardHeader,
  CFormGroup,
  CInputGroup,
  CInputGroupPrepend,
  CInputGroupText,
  CInput,
  CInputGroupAppend,
  CInvalidFeedback,
  CCardFooter,
  CButton,
  CSelect,
  CForm,
} from '@coreui/react'
import { albumsApi, artistsApi, providersApi, authApi, genresApi, configsApi, contractsApi } from '../../../api'
import { ACTIONS, MODELS, businessTypes } from '../../../constants'
import SongTable from './SongTable'
import UploadImages from '../../../containers/ImageUpload'
import MultipleSelectDropdown from '../../../containers/MultipleSelectDropdown'
import GroupSelect from '../../../containers/GroupSelect'
import AutocompleteInput from '../../../containers/AutocompleteInput'

const AlbumForm = (props) => {
  const { match: {params: { id }}, history } = props
  const [newAlbum, setNewAlbum] = useState({})
  const [artists, setArtists] = useState({
    artists: [],
    composers: [],
    authors: []
  })
  const [configs, setConfigs] = useState({
    imageMaxWidth: 1024,
    imageMaxHeight: 768,
  })
  const [errors, setErrors] = useState({})
  const [images, setImages] = useState([])
  const [files, setFiles] = useState([])
  const [songs, setSongs] = useState([])
  const [genres, setGenres] = useState({})
  const [isLoading, setLoading] = useState(false)
  const [providers, setProviders] = useState([])
  const [contracts, setContracts] = useState([])
  const [permissions, setPermissions] = useState({
    editableAlbum: false,
    editableProvider: false
  })

  useEffect(() => {
    const fetchPermissions = async () => {
      const editableProvider = await authApi.checkPermission(MODELS.PROVIDER_DETAILS, ACTIONS.EDIT)
      const editableAlbum = await authApi.checkPermission(MODELS.ALBUM_DETAILS, ACTIONS.EDIT)

      setPermissions({ editableAlbum, editableProvider })
    }
    const fetchConfigs = async () => {
      const res = await configsApi.getConfigs()
      if (!res.isError) {
        setConfigs(res.data.configs)
      }
    }

    fetchConfigs()

    fetchPermissions()
  }, [])


  const onBack = () => {
    props.history.goBack()
  }

  useEffect(() => {
    const fetchProviders = async () => {
      const res = await providersApi.getListForSelect()
      if (!res.isError && res?.data?.providers) {
        setProviders(res.data.providers)
      }
    }
    const fetchContracts = async () => {
      const res = await contractsApi.getListForSelect()
      if (!res.isError && res?.data?.contracts) {
        setContracts(res.data.contracts)
      }
    }

    const formatGenresByGroup = (genres) => {
      const genreGroups = {}
      const parents = {}
      genres.forEach(g => {
        if (g.level === 0) {
          parents[g.id] = g
          genreGroups[g.category] = []
          return
        }
      })
      genres.forEach((g) => {
        if (g.level === 0) return
        let parentCategory = parents[g.parentId]?.category
        if (!parentCategory) return
        genreGroups[parentCategory]?.push(g)
      })
      return genreGroups
    }

    const fetchGenres = async () => {
      const res = await genresApi.getListForSelect()
      if (!res.isError) {
        return setGenres(formatGenresByGroup(res.data.genres))
      }
    }
    fetchGenres()
    fetchProviders()
    fetchContracts()
  }, [])


  useEffect(() => {
    const fetchAlbum = async (id) => {
      if (!id) return
      setLoading(true)
      const res = await albumsApi.getOne(id)
      if (res.isError || !res?.data?.album) {
        setErrors({general: res.errorMessage || 'Album not found'})
        setTimeout(() => history.push('/albums'), 3000)
        return
      }
      const resAlbum = res.data.album
      const images = resAlbum.images.map(p => p?.filePath)
      resAlbum.artists = resAlbum.artists.map(a => a.artistId)
      resAlbum.composers = resAlbum.composers.map(a => a.artistId)
      resAlbum.authors = resAlbum.authors.map(a => a.artistId)
      resAlbum.genre = resAlbum.genres?.[0] ? {label: resAlbum.genres[0].category, value: resAlbum.genres[0].id} : null
      resAlbum.contractId = resAlbum.contract ? {label: resAlbum.contract.id, value: resAlbum.contract.id} : null
      resAlbum.artistName = resAlbum.mainArtist?.name || ''
      resAlbum.contractStartDate = resAlbum.contract?.startDate?.slice(0, 10)
      resAlbum.contractEndDate = resAlbum.contract?.endDate?.slice(0, 10)

      setNewAlbum({...resAlbum})
      setSongs(resAlbum.media || [])
      setImages(images)
      setFiles(images)
      setLoading(false)
    }

    fetchArtirts()
    if (id) fetchAlbum(id)
    else {
      let userInfo = localStorage.getItem('userProfile')
      try {
        userInfo = JSON.parse(userInfo)
        setNewAlbum({...newAlbum, providerId: userInfo.providerId, initiatorName: userInfo.name})
      }
      catch (err) {
        console.log({ERROR: err.message})
      }
    }
  }, [id, history])

  const fetchArtirts = async () => {
    let res = await artistsApi.fetchArtistsAndGroup()
    if (!res.isError) {
      setArtists(res.data)
    }
  }

  const onSelectArtist = (name,  values ) => {
    setNewAlbum({...newAlbum, [name]: values})
  }

  const onUploadImages = (imageUrls, inputFiles) => {
    setImages([...images, ...imageUrls])
    setFiles([...files, ...inputFiles])
  }

  const onRemoveImage = (pos) => {
    setImages(images.filter((_, i) => i !== pos))
    setFiles([...files.filter((_, i) => i !== pos)])
  }

  const addMedia = (albums) => {
    setSongs([...songs, ...albums])
  }

  const onChangeValue = ({target: { name, value }}) => {
    setErrors({...errors, [name]: '', general: ''})
    const updateAlbum = {
      [name]: value
    }
    if (name === 'contractId') {
      value = value.value
      let selectedContract = contracts.find(c => c.id === value)
      const contractStart = selectedContract?.startDate?.slice(0, 10)
      const contractEnd = selectedContract?.endDate?.slice(0, 10)
      updateAlbum.contractStartDate = contractStart
      updateAlbum.contractEndDate = contractEnd
    }
    setNewAlbum({...newAlbum, ...updateAlbum})
  }

  const generateRelationFiles = (album, files) => {
    const isNotUpdateImageIds = files.map((file, idx) => typeof file === 'string' ? album.images[idx].id : "").filter(id => !!id)
    const submitAlbum = {...album, isNotUpdateImageIds}
    submitAlbum.images = files.filter(f => typeof f !== 'string')
    submitAlbum.genres = submitAlbum.genre ? [submitAlbum.genre?.value] : []
    submitAlbum.contractId = submitAlbum?.contractId.value || null
    return submitAlbum
  }

  const onSubmit = async () => {
    let validateErrors = {};
    if (!newAlbum.name) {
      validateErrors['name'] = 'Album name is required!';
    }
    if (newAlbum.artistName) {
      const artist = artists?.artists?.find(a => a.name.trim() === newAlbum.artistName.trim())
      if (!artist) {
        validateErrors['artistName'] = 'Please select main artist in dropdown!'
      } else {
        newAlbum.artistId = artist.id
      }
    } else {
      delete newAlbum.artistId
    }
    if (newAlbum.contractId?.value) {
      const contract = contracts?.find(a => a.id === newAlbum.contractId.value.trim())
      if (!contract) {
        validateErrors['contractId'] = 'Please select a contract in suggestion list!'
      }
    } else {
      delete newAlbum.contractId
    }
    if (Object.values(validateErrors).length) return setErrors(validateErrors)
    setLoading(true)
    debugger
    if (id) {
      const editAlbum = generateRelationFiles(newAlbum, files)
      editAlbum.media = songs.map(s => s.id)
      let res = await albumsApi.edit(id, editAlbum)

      if (res.isError) {
        setLoading(false)
        return setErrors({general: res.errorMessage})
      }

      return props.history.push('/albums')
    }

    let res = await albumsApi.create({
      ...generateRelationFiles(newAlbum, files),
      status: 'active',
      media: songs.map(s => s.id)
    })

    if (res.isError) {
      setLoading(false)
      return setErrors({general: res.errorMessage})
    }
    props.history.push('/albums')
  }

  const removeMedium = (index) => {
    setSongs([...songs.slice(0, index), ...songs.slice(index + 1)])
  }

  return (
    <CCard>
      <CCardHeader align="center">
        <h1>{id ? 'Edit Album' : 'Register Album'}</h1>
      </CCardHeader>
      <CCardBody>
        <CForm className="form-horizontal" formAction="">
          <div className="row justify-content align-items-center">
            <div className="col-4">
              <UploadImages disabled={!permissions.editableAlbum || isLoading} images={images} onUploadImages={onUploadImages} onRemoveImage={onRemoveImage} maxWidth={configs.imageMaxWidth} maxHeight={configs.imageMaxHeight}/>
            </div>
            <div className="col-8">
              <CFormGroup className="col-12 row">
                <CInputGroup>
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "150px"}} >Name</CInputGroupText>
                  </CInputGroupPrepend>
                  <CInput
                    onChange={onChangeValue}
                    defaultValue={newAlbum ? newAlbum.name : ''} type="text"
                    id="name"
                    name="name"
                    invalid={!!errors["name"]}
                    disabled={!permissions.editableAlbum || isLoading}
                  />
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                  {!!errors["name"] ? <CInvalidFeedback>{errors["name"]}</CInvalidFeedback> : ""}
                </CInputGroup>
              </CFormGroup>
              <CFormGroup className="col-12 row">
                <CInputGroup>
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "150px"}} >Artist</CInputGroupText>
                  </CInputGroupPrepend>
                  <MultipleSelectDropdown
                    options={artists.artists}
                    values={newAlbum.artists || []}
                    valueKey="id"
                    textKey="name"
                    name="artistSelect"
                    onSelect={(values) => onSelectArtist('artists', values)}
                    onRemove={() => onSelectArtist('artists', [])}
                    disabled={!permissions.editableAlbum || isLoading}
                  />
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                  {!!errors["artistName"] ? <CInvalidFeedback>{errors["artistName"]}</CInvalidFeedback> : ""}
                </CInputGroup>
              </CFormGroup>
              <AutocompleteInput
                className="col-12 row"
                prependInput={
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "150px"}} >Main Artist</CInputGroupText>
                  </CInputGroupPrepend>
                }
                appendInput={
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                }
                error={errors["artistName"]}
                name="artistName"
                onChange={onChangeValue}
                suggestions={artists?.artists?.map(a => a.name)}
                value={newAlbum.artistName}
                disabled={permissions.editableAlbum || isLoading}
              />
              <CFormGroup className="col-12 row">
                <CInputGroup>
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "150px"}} >Provider</CInputGroupText>
                  </CInputGroupPrepend>
                    <CSelect
                      onChange={onChangeValue}
                      value={newAlbum ? newAlbum.providerId : ''}
                      type="text"
                      id="providerId"
                      name="providerId"
                      invalid={!!errors["providerId"]}
                      disabled={!permissions.editableAlbum || !permissions.editableProvider || isLoading}
                    >
                      <option value="">Select Provider</option>
                      {providers.map((provider, pos) => (
                        <option value={provider.id} key={pos}>{provider.name}</option>
                      ))}
                    </CSelect>
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                  {!!errors["providerId"] ? <CInvalidFeedback>{errors["providerId"]}</CInvalidFeedback> : ""}
                </CInputGroup>
              </CFormGroup>
              <CFormGroup className="col-12 row">
                <CInputGroup>
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "150px"}} >Genres</CInputGroupText>
                  </CInputGroupPrepend>
                  <GroupSelect
                    className="multiple-select"
                    name="genre"
                    optionGroups={genres}
                    textField="category"
                    valueField="id"
                    value={newAlbum.genre}
                    onChange={onChangeValue}
                  />
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                  {!!errors["genre"] ? <CInvalidFeedback>{errors["genre"]}</CInvalidFeedback> : ""}
                </CInputGroup>
              </CFormGroup>
              <CFormGroup className="col-12 row">
                <CInputGroup>
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "150px"}} >Type</CInputGroupText>
                  </CInputGroupPrepend>
                    <CSelect
                      onChange={onChangeValue}
                      value={newAlbum ? newAlbum.type : ''}
                      disabled={!permissions.editableAlbum || isLoading}
                      type="text"
                      id="type"
                      name="type"
                      invalid={!!errors["type"]}
                    >
                      <option value="">Select Type</option>
                      {businessTypes.map((type, pos) => (
                        <option value={type.value} key={pos}>{type.name}</option>
                      ))}
                    </CSelect>
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                  {!!errors["type"] ? <CInvalidFeedback>{errors["type"]}</CInvalidFeedback> : ""}
                </CInputGroup>
              </CFormGroup>
              <CFormGroup className="col-12 row">
                <CInputGroup>
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "150px"}} >Project Number</CInputGroupText>
                  </CInputGroupPrepend>
                  <CInput
                    onChange={onChangeValue}
                    defaultValue={newAlbum ? newAlbum.projectNumber : ''}
                    type="text"
                    id="projectNumber"
                    name="projectNumber"
                    className="no-icon"
                    disabled={!permissions.editableAlbum || isLoading}
                  />
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                </CInputGroup>
              </CFormGroup>
              <CFormGroup className="col-12 row">
                <CInputGroup>
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "150px"}} >Label</CInputGroupText>
                  </CInputGroupPrepend>
                  <CInput
                    onChange={onChangeValue}
                    defaultValue={newAlbum ? newAlbum.label : ''}
                    type="text"
                    id="label"
                    name="label"
                    className="no-icon"
                    disabled={!permissions.editableAlbum || isLoading}
                  />
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                </CInputGroup>
              </CFormGroup>
              <CFormGroup className="col-12 row">
                <CInputGroup>
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "150px"}} >Repertoire Owner</CInputGroupText>
                  </CInputGroupPrepend>
                  <CInput
                    onChange={onChangeValue}
                    defaultValue={newAlbum ? newAlbum.repertoireOwner : ''}
                    type="text"
                    id="repertoireOwner"
                    name="repertoireOwner"
                    className="no-icon"
                    disabled={!permissions.editableAlbum || isLoading}
                  />
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                </CInputGroup>
              </CFormGroup>
              <CFormGroup className="col-12 row">
                <CInputGroup>
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "150px"}} >Maintainer Owner</CInputGroupText>
                  </CInputGroupPrepend>
                  <CInput
                    onChange={onChangeValue}
                    defaultValue={newAlbum ? newAlbum.maintainerOwner : ''}
                    type="text"
                    id="maintainerOwner"
                    name="maintainerOwner"
                    className="no-icon"
                    disabled={!permissions.editableAlbum || isLoading}
                  />
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                </CInputGroup>
              </CFormGroup>
              <CFormGroup className="col-12 row">
                <CInputGroup>
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "150px"}} >Initiator</CInputGroupText>
                  </CInputGroupPrepend>
                  <CInput
                    onChange={onChangeValue}
                    defaultValue={newAlbum ? newAlbum.initiatorName : ''}
                    type="text"
                    id="initiatorName"
                    name="initiatorName"
                    className="no-icon"
                    readOnly="true"
                  />
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                </CInputGroup>
              </CFormGroup>
              <CFormGroup className="col-12 row">
                <CInputGroup>
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "150px"}} >Contract Number</CInputGroupText>
                  </CInputGroupPrepend>
                  <GroupSelect
                    className="multiple-select"
                    name="contractId"
                    optionGroups={contracts}
                    textField="id"
                    valueField="id"
                    value={newAlbum.contractId}
                    onChange={onChangeValue}
                  />
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                </CInputGroup>
              </CFormGroup>
              <CFormGroup className="col-12 row">
                <CInputGroup>
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "150px"}} >Contract Start Date</CInputGroupText>
                  </CInputGroupPrepend>
                  <CInput
                    onChange={onChangeValue}
                    defaultValue={newAlbum ? newAlbum.contractStartDate : ''}
                    type="text"
                    id="contractStartDate"
                    readOnly
                    className="no-icon"
                  />
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                </CInputGroup>
              </CFormGroup>
              <CFormGroup className="col-12 row">
                <CInputGroup>
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "150px"}} >Contract End Date</CInputGroupText>
                  </CInputGroupPrepend>
                  <CInput
                    onChange={onChangeValue}
                    defaultValue={newAlbum ? newAlbum.contractEndDate : ''}
                    type="text"
                    id="contractEndDate"
                    readOnly
                    className="no-icon"
                  />
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                </CInputGroup>
              </CFormGroup>
              <CFormGroup className="col-12 row">
                <CInputGroup className="mb-4">
                  {!!errors["general"] ? <CInput hidden invalid={!!errors["general"]} /> : ""}
                  {!!errors["general"] ? <CInvalidFeedback>{errors["general"]}</CInvalidFeedback> : ""}
                </CInputGroup>
              </CFormGroup>
            </div>
          </div>
          <div className="col-12 justify-content-center">
            <SongTable disabled={!permissions.editableAlbum} className="" songs={songs} addMedia={addMedia} removeMedium={removeMedium} />
          </div>
          <CCardFooter className="row justify-content-around pt-3 pb-0">
            <div className="col-8 row justify-content-around">
              <CButton
                type="button"
                className="btn-pill col-4"
                size="lg"
                color="primary"
                onClick={onSubmit}
                disabled={!permissions.editableAlbum || isLoading}
              >
                {!!isLoading && <span class="spinner-border spinner-border-sm" role="status" aria-hidden="true"></span>}
                {isLoading ? "Submitting..." : "Submit"}
              </CButton>
              <CButton disabled={isLoading} type="button" className="btn-pill col-4" size="lg" color="dark" onClick={onBack}>Back</CButton>
            </div>
          </CCardFooter>
        </CForm>
      </CCardBody>
    </CCard>
  )
}

export default AlbumForm