import React, { useEffect, useState } from 'react'
import {
  CCard,
  CCardBody,
  CCardHeader,
  CFormGroup,
  CInputGroup,
  CInputGroupPrepend,
  CInputGroupText,
  CInput,
  CInputGroupAppend,
  CInvalidFeedback,
  CSelect,
  CCardFooter,
  CButton,
  CForm,
  CTextarea,
} from '@coreui/react'

import { artistsApi, authApi, genresApi, configsApi } from '../../../api'
import { MODELS, ACTIONS, CountryList } from '../../../constants'
import MultipleSelectDropdown from '../../../containers/MultipleSelectDropdown'
import GroupSelect from '../../../containers/GroupSelect'
import AutocompleteInput from '../../../containers/AutocompleteInput'

const professionConst = {
  'artist': {label: "Artist", value: 'Artist'},
  'author': {label: "Author", value: 'Author'},
  'composer': {label: "Composer", value: 'Composer'}
}

const ArtistForm = (props) => {
  const { match: {params: { id }}, history } = props
  const [genres, setGenres] = useState([])
  const [configs, setConfigs] = useState({
    imageMaxWidth: 1024,
    imageMaxHeight: 768,
  })

  const [newArtist, setNewArtist] = useState({
    name: "",
    gender: "",
    professions: [],
    country: "",
    story: ""
  })
  const [errors, setErrors] = useState({})
  const [image, setImage] = useState(null)
  const [imageFile, setImageFile] = useState(null)
  const [isLoading, setLoading] = useState(false)
  const [permissions, setPermissions] = useState({
    editableArtist: false
  })

  useEffect(() => {
    const fetchPermissions = async () => {
      const editableArtist = await authApi.checkPermission(MODELS.ARTIST_DETAILS, ACTIONS.EDIT)

      setPermissions({ editableArtist })
    }

    const formatGenresByGroup = (genres) => {
      const genreGroups = {}
      const parents = {}
      genres.forEach(g => {
        if (g.level === 0) {
          parents[g.id] = g
          genreGroups[g.category] = []
          return
        }
      })
      genres.forEach((g) => {
        if (g.level === 0) return
        let parentCategory = parents[g.parentId]?.category
        if (!parentCategory) return
        genreGroups[parentCategory]?.push(g)
      })
      return genreGroups
    }

    const fetchGenres = async () => {
      const res = await genresApi.getListForSelect()
      if (!res.isError) {
        return setGenres(formatGenresByGroup(res.data.genres))
      }
    }
    const fetchConfigs = async () => {
      const res = await configsApi.getConfigs()
      if (!res.isError) {
        setConfigs(res.data.configs)
      }
    }


    fetchPermissions()
    fetchGenres()
    fetchConfigs()
  }, [])

  useEffect(() => {
    const fetchArtist = async () => {
      if (!id) return
      setLoading(true)
      const res = await artistsApi.getOne(id)
      setLoading(false)
      if (res.isError || !res?.data?.artist) {
        setErrors({general: res.errorMessage || 'Artist not found'})
        setTimeout(() => history.push('/artists'), 3000)
        return
      }
      const resArtist = res.data.artist
      let image = resArtist.profileImage?.filePath
      delete resArtist.profileImage
      resArtist.genre = resArtist.genres?.[0] ? {label: resArtist.genres[0].category, value: resArtist.genres[0].id} : null
      setNewArtist({
        ...resArtist,
        professions: resArtist.professions.map(p => professionConst[p.toLowerCase()]?.value),
      })
      setImage(image)
    }

    fetchArtist()
  }, [id, history])
  const onBack = () => {
    props.history.goBack()
  }

  const onUploadImage = (img) => {
    setImage(img)
  }

  const onChangeImageFile = (img) => {
    setImageFile(img)
  }

  const onChangeValue = ({target: { name, value }}) => {
    setErrors({...errors, [name]: '', general: ''})
    setNewArtist({...newArtist, [name]: value})
  }

  const onChangeProfessions = (professions) => {
    setNewArtist({...newArtist, professions})
  }

  const onRemoveProfessions = () => {
    return onChangeProfessions([])
  }

  const onSubmit = async () => {
    let validateErrors = {}
    if (!newArtist.fullName) {
      validateErrors['fullName'] = 'fullName is required!'
    }
    if (!newArtist.firstName) {
      validateErrors['firstName'] = 'firstName is required!'
    }
    if (!newArtist.lastName) {
      validateErrors['lastName'] = 'lastName is required!'
    }
    if (!newArtist.professions.length) {
      validateErrors['professions'] = 'Must select at least one profession!'
    }
    newArtist.genres = newArtist.genre ? [newArtist.genre?.value] : []

    if (Object.values(validateErrors).length) return setErrors(validateErrors)
    setLoading(true)
    if (id) {
      if (imageFile) {
        newArtist.profileImage = imageFile
      }
      let res = await artistsApi.edit(id, {...newArtist})

      if (res.isError) {
        setLoading(false)
        return setErrors({general: res.errorMessage})
      }

      return props.history.push('/artists')
    }

    let response = await artistsApi.create({
      ...newArtist,
      status: 'active',
      profileImage: imageFile
    })
    if (response.isError) {
      setLoading(false)
      return setErrors({general: response.errorMessage})
    }

    props.history.push('/artists')
  }

  return (
    <CCard>
      <CCardHeader align="center">
        <h1>{id ? 'Edit Artist' : 'Register Artist'}</h1>
      </CCardHeader>
      <CCardBody>
        <CForm className="form-horizontal" formAction="">
          <div className="row justify-content">
            <div className="col-4 row justify-content-center">
              <UploadImage
                disabled={!permissions.editableArtist || isLoading}
                image={image}
                onUploadImage={onUploadImage}
                name="Avatar"
                onChangeImageFile={onChangeImageFile}
                maxWidth={configs.imageMaxWidth}
                maxHeight={configs.imageMaxHeight}
              />
            </div>
            <div className="col-8 row justify-content-center">
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "120px"}} >First Name</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      onChange={onChangeValue}
                      value={newArtist ? newArtist.firstName : ''}
                      disabled={!permissions.editableArtist || isLoading}
                      type="text"
                      id="name"
                      name="firstName"
                      invalid={!!errors["firstName"]}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors["firstName"] ? <CInvalidFeedback>{errors["firstName"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              </div>
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "120px"}} >Last Name</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      onChange={onChangeValue}
                      value={newArtist ? newArtist.lastName : ''}
                      disabled={!permissions.editableArtist || isLoading}
                      type="text"
                      id="name"
                      name="lastName"
                      invalid={!!errors["lastName"]}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors["lastName"] ? <CInvalidFeedback>{errors["lastName"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              </div>
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "120px"}} >Full Name</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      onChange={onChangeValue}
                      value={newArtist ? newArtist.fullName : ''}
                      disabled={!permissions.editableArtist || isLoading}
                      type="text"
                      id="name"
                      name="fullName"
                      invalid={!!errors["fullName"]}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors["fullName"] ? <CInvalidFeedback>{errors["fullName"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              </div>
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "120px"}} >History</CInputGroupText>
                    </CInputGroupPrepend>
                    <CTextarea
                      onChange={onChangeValue}
                      value={newArtist ? newArtist.history : ''}
                      disabled={!permissions.editableArtist || isLoading}
                      type="text"
                      id="history"
                      name="history"
                      invalid={!!errors["history"]}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-book-open" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors["history"] ? <CInvalidFeedback>{errors["history"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              </div>
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "120px"}} >Gender</CInputGroupText>
                    </CInputGroupPrepend>
                    <CSelect
                      onChange={onChangeValue}
                      value={newArtist ? newArtist.gender : ''}
                      disabled={!permissions.editableArtist || isLoading}
                      type="text"
                      id="gender"
                      name="gender"
                      invalid={!!errors["gender"]}
                    >
                      <option value="">Select Gender</option>
                      <option value="male">Male</option>
                      <option value="female">Female</option>
                    </CSelect>
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-venus-mars" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors["gender"] ? <CInvalidFeedback>{errors["gender"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              </div>
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "120px"}} >Date Of Birth</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      onChange={onChangeValue}
                      defaultValue={newArtist ? newArtist.dob : ''}
                      type="date"
                      id="dob"
                      name="dob"
                      className="no-icon"
                      disabled={!permissions.editableArtist || isLoading}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                  </CInputGroup>
                </CFormGroup>
              </div>
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "120px"}} >Professions</CInputGroupText>
                    </CInputGroupPrepend>
                    <MultipleSelectDropdown
                      options={Object.values(professionConst)} // Options to display in the dropdown
                      hasSelectAll={false}
                      values={newArtist.professions || []} // Preselected value to persist in dropdown
                      disabled={!permissions.editableArtist || isLoading}
                      // onSelect={this.onSelect} // Function will trigger on select event
                      onRemove={onRemoveProfessions} // Function will trigger on remove event
                      onSelect={onChangeProfessions}
                      displayValue="" // Property name to display in the dropdown options
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-user-tie" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors["professions"] ? <CInvalidFeedback>{errors["professions"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              </div>
              <div className="col-12">
              <CFormGroup>
                <CInputGroup>
                  <CInputGroupPrepend>
                    <CInputGroupText style={{ width: "120px"}} >Genres</CInputGroupText>
                  </CInputGroupPrepend>
                  <GroupSelect
                    className="multiple-select"
                    name="genre"
                    optionGroups={genres}
                    textField="category"
                    valueField="id"
                    value={newArtist.genre}
                    onChange={onChangeValue}
                  />
                  <CInputGroupAppend>
                    <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                  </CInputGroupAppend>
                  {!!errors["genre"] ? <CInvalidFeedback>{errors["genre"]}</CInvalidFeedback> : ""}
                </CInputGroup>
              </CFormGroup>
              </div>
              <div className="col-12">
                <AutocompleteInput
                  prependInput={
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "120px"}} >Country</CInputGroupText>
                    </CInputGroupPrepend>
                  }
                  appendInput={
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-globe-asia" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                  }
                  error={errors["country"]}
                  name="country"
                  onChange={onChangeValue}
                  suggestions={CountryList}
                  value={newArtist.country}
                />
              </div>
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup className="mb-4">
                    {!!errors["general"] ? <CInput hidden invalid={!!errors["general"]} /> : ""}
                    {!!errors["general"] ? <CInvalidFeedback>{errors["general"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              </div>
            </div>
          </div>
          <CCardFooter className="row justify-content-around pt-3 pb-0">
            <div className="col-6 row justify-content-around">
              <CButton disabled={!permissions.editableArtist || isLoading} type="button" className="btn-pill col-4" size="lg" color="primary" onClick={onSubmit}>
                {!!isLoading && <span class="spinner-border spinner-border-sm" role="status" aria-hidden="true"></span>}
                {isLoading ? "Submitting..." : "Submit"}
              </CButton>
              <CButton disabled={isLoading} type="button" className="btn-pill col-4" size="lg" color="dark" onClick={onBack}>Back</CButton>
            </div>
          </CCardFooter>
        </CForm>
      </CCardBody>
    </CCard>
  )
}

const UploadImage = (props) => {
  const [imgPreview, setImgPreview] = useState('')
  const [error, setError] = useState('')

  const chooseImage = (e) => {
    if (!e.target.files || !e.target.files[0]) return
    const img = new Image();
    var objectUrl = URL.createObjectURL(e.target.files[0]);
    img.onload = function () {
      // URL.revokeObjectURL(objectUrl);
      if (this.width > props.maxWidth || this.height > props.maxHeight) {
        return setError(`Image size must less than ${props.maxWidth}*${props.maxHeight}`)
      }
      setError("")
      setImgPreview(objectUrl)
      if (props.onUploadImage) {
        props.onUploadImage(objectUrl)
      }
      if (props.onChangeImageFile) {
        props.onChangeImageFile(e.target.files[0])
      }
    };
    img.src = objectUrl;
  }
  const chooseFileUpload = () => document.getElementById('uploadImage').click()
  return (
    [<div style={{ cursor: 'pointer' }}>
      <input
        hidden
        id="uploadImage"
        type="file"
        accept="image/x-png,image/gif,image/jpeg"
        onChange={chooseImage}
        disabled={props.disabled}
        />
      <img
        src={props.image || imgPreview || 'https://www.seekpng.com/png/detail/110-1100707_person-avatar-placeholder.png'}
        alt="Profile"
        onClick={chooseFileUpload}
        width="100%"
        style={{maxHeight: '250px', maxWidth: '300px'}}
      />
      <div>
        <span style={{
          position: 'relative',
          left: 'calc(50% - 95px)',
          top: '-150px',
          fontSize: '2rem',
          color: '#adadad',
          display: 'block',
        }}
        onClick={chooseFileUpload}
        hidden={!!props.image || !!imgPreview}
        >Profile Image</span>
      </div>
    </div>,
    <CInputGroup className="">
      {!!error ? <CInput hidden invalid={!!error} /> : ""}
      {!!error ? <CInvalidFeedback>{error}</CInvalidFeedback> : ""}
    </CInputGroup>
    ]
  )
}

export default ArtistForm