import React, { useEffect, useState } from 'react'
import {
  CCard,
  CCardBody,
  CCardHeader,
  CFormGroup,
  CInputGroup,
  CInputGroupPrepend,
  CInputGroupText,
  CInput,
  CInputGroupAppend,
  CInvalidFeedback,
  CSelect,
  CCardFooter,
  CButton,
  CForm,
  CTooltip,
} from '@coreui/react'

import { contractsApi, authApi, providersApi } from '../../../api'
import { MODELS, ACTIONS } from '../../../constants'

const ContractForm = (props) => {
  const { match: {params: { id }}, history } = props
  const [newContract, setNewContract] = useState({
    id: "",
    name: "",
    startDate: "",
    endDate: "",
    providerId: "",
  })
  const [errors, setErrors] = useState({})
  const [providers, setProviders] = useState([])
  const [pdfFile, setPdfFile] = useState(null)
  const [isLoading, setLoading] = useState(false)
  const [permissions, setPermissions] = useState({
    editableContract: false,
    editableProvider: false,
  })

  useEffect(() => {
    const fetchPermissions = async () => {
      const editableContract = await authApi.checkPermission(MODELS.MEDIUM_DETAILS, ACTIONS.EDIT)
      const editableProvider = await authApi.checkPermission(MODELS.PROVIDER_DETAILS, ACTIONS.EDIT)

      setPermissions({ editableContract, editableProvider })
    }
    const fetchProviders = async () => {
      const res = await providersApi.getListForSelect()
      if (!res.isError && res?.data?.providers) {
        setProviders(res.data.providers)
      }
    }

    fetchPermissions()
    fetchProviders()
  }, [])

  useEffect(() => {
    const fetchContract = async () => {
      setLoading(true)
      const res = await contractsApi.getOne(id)
      setLoading(false)
      if (res.isError || !res?.data?.contract) {
        setErrors({general: res.errorMessage || 'Contract not found'})
        setTimeout(() => history.push('/contracts'), 3000)
        return
      }
      const resContract = res.data.contract
      setNewContract({
        ...resContract,
      })
    }

    if (id) fetchContract()
    else {
      let userInfo = localStorage.getItem('userProfile')
      try {
        userInfo = JSON.parse(userInfo)
        setNewContract({...newContract, providerId: userInfo.providerId})
      }
      catch (err) {
        console.log({ERROR: err.message})
      }
    }
  }, [id, history])
  const onBack = () => {
    props.history.goBack()
  }

  const openUpload = (id) => {
    document.getElementById(id).click()
  }

  const uploadFile = ({ target: { files: inputFiles }}) => {
    if (inputFiles && inputFiles[0]) {
      setPdfFile(inputFiles[0])
    }
  }

  const removeFile = (e, fileField) => {
    e.stopPropagation()
    setPdfFile(null)
    document.getElementById(fileField).value = null
  }

  const onChangeValue = ({target: { name, value }}) => {
    setErrors({...errors, [name]: '', general: ''})
    setNewContract({...newContract, [name]: value})
  }

  const onSubmit = async () => {
    let validateErrors = {}
    if (!newContract.name) {
      validateErrors['name'] = 'name is required!'
    }
    if (!newContract.id) {
      validateErrors['id'] = 'Contract number is required!'
    }
    if (!newContract.startDate) {
      validateErrors['startDate'] = 'Start date is required!'
    }

    if (Object.values(validateErrors).length) return setErrors(validateErrors)
    setLoading(true)
    if (id) {
      if (pdfFile) {
        newContract.pdf = pdfFile
      }
      let res = await contractsApi.edit(id, {...newContract})

      if (res.isError) {
        setLoading(false)
        return setErrors({general: res.errorMessage})
      }

      return props.history.push('/contracts')
    }

    let response = await contractsApi.create({
      ...newContract,
      status: 'active',
      pdf: pdfFile
    })
    if (response.isError) {
      setLoading(false)
      return setErrors({general: response.errorMessage})
    }

    props.history.push('/contracts')
  }

  return (
    <CCard>
      <CCardHeader align="center">
        <h1>{id ? 'Edit Contract' : 'Create Contract'}</h1>
      </CCardHeader>
      <CCardBody>
        <CForm className="form-horizontal" formAction="">
          <div className="row justify-content-center">
            <div className="col-8 row">
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "150px"}} >Contract Number</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      onChange={onChangeValue}
                      value={newContract ? newContract.id : ''}
                      disabled={!permissions.editableContract || isLoading}
                      type="text"
                      id="name"
                      name="id"
                      invalid={!!errors["id"]}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors["id"] ? <CInvalidFeedback>{errors["id"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              </div>
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "150px"}} >Name</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      onChange={onChangeValue}
                      value={newContract ? newContract.name : ''}
                      disabled={!permissions.editableContract || isLoading}
                      type="text"
                      id="name"
                      name="name"
                      invalid={!!errors["name"]}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors["name"] ? <CInvalidFeedback>{errors["name"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              </div>
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "150px"}} >Provider</CInputGroupText>
                    </CInputGroupPrepend>
                      <CSelect
                        onChange={onChangeValue}
                        value={newContract ? newContract.providerId : ''}
                        disabled={!permissions.editableContract || !permissions.editableProvider || isLoading}
                        type="text"
                        id="providerId"
                        name="providerId"
                        invalid={!!errors["providerId"]}
                      >
                        <option value="">Select Provider</option>
                        {providers.map((provider, pos) => (
                          <option value={provider.id} key={pos}>{provider.name}</option>
                        ))}
                      </CSelect>
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors["providerId"] ? <CInvalidFeedback>{errors["providerId"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              </div>
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "150px"}} >Contract Start Date</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      onChange={onChangeValue}
                      defaultValue={newContract ? newContract.startDate : ''}
                      type="date"
                      id="startDate"
                      name="startDate"
                      className="no-icon"
                      disabled={!permissions.editableContract || isLoading}
                      invalid={!!errors["startDate"]}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors["startDate"] ? <CInvalidFeedback>{errors["startDate"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              </div>
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "150px"}} >Contract End Date</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      onChange={onChangeValue}
                      defaultValue={newContract ? newContract.endDate : ''}
                      type="date"
                      id="endDate"
                      name="endDate"
                      className="no-icon"
                      disabled={!permissions.editableContract || isLoading}
                      invalid={!!errors["endDate"]}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors["endDate"] ? <CInvalidFeedback>{errors["endDate"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              </div>
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CTooltip content={`PDF Contract`} placement="top-start">
                        <CInputGroupText style={{ width: "150px"}}>Contract File <i className="far fa-question-circle pl-2"/></CInputGroupText>
                      </CTooltip>
                    </CInputGroupPrepend>
                    <div className={"form-control file-input" + (errors["pdfFile"] ? ' is-invalid' : '')} onClick={() => openUpload('pdf')} style={{ cursor: 'pointer' }}>
                      {pdfFile?.name || newContract?.pdf?.name || 'Choose File...'}
                      {!!pdfFile &&
                        <CButton className="btn-remove-file" onClick={(e) => removeFile(e, 'pdf')}>
                          <svg width="24" height="24" fill="none" stroke="currentColor" stroke-width="2" class="dropdown-search-clear-icon gray"><line x1="18" y1="6" x2="6" y2="18"></line><line x1="6" y1="6" x2="18" y2="18"></line></svg>
                        </CButton>
                      }
                    </div>
                    <input disabled={!permissions.editableContract || isLoading} type="file" accept="application/pdf" id="pdf" name="pdf" hidden onChange={uploadFile}/>
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-file-pdf" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors["pdfFile"] ? <CInvalidFeedback>{errors["pdfFile"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              </div>
              <div className="col-12">
                <CFormGroup>
                  <CInputGroup className="mb-4">
                    {!!errors["general"] ? <CInput hidden invalid={!!errors["general"]} /> : ""}
                    {!!errors["general"] ? <CInvalidFeedback>{errors["general"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              </div>
            </div>
          </div>
          <CCardFooter className="row justify-content-around pt-3 pb-0">
            <div className="col-6 row justify-content-around">
              <CButton disabled={!permissions.editableContract || isLoading} type="button" className="btn-pill col-4" size="lg" color="primary" onClick={onSubmit}>
                {!!isLoading && <span class="spinner-border spinner-border-sm" role="status" aria-hidden="true"></span>}
                {isLoading ? "Submitting..." : "Submit"}
              </CButton>
              <CButton disabled={isLoading} type="button" className="btn-pill col-4" size="lg" color="dark" onClick={onBack}>Back</CButton>
            </div>
          </CCardFooter>
        </CForm>
      </CCardBody>
    </CCard>
  )
}

const UploadImage = (props) => {
  const [imgPreview, setImgPreview] = useState('')
  const [error, setError] = useState('')

  const chooseImage = (e) => {
    if (!e.target.files || !e.target.files[0]) return
    const img = new Image();
    var objectUrl = URL.createObjectURL(e.target.files[0]);
    img.onload = function () {
      // URL.revokeObjectURL(objectUrl);
      if (this.width > props.maxWidth || this.height > props.maxHeight) {
        return setError(`Image size must less than ${props.maxWidth}*${props.maxHeight}`)
      }
      setError("")
      setImgPreview(objectUrl)
      if (props.onUploadImage) {
        props.onUploadImage(objectUrl)
      }
      if (props.onChangeImageFile) {
        props.onChangeImageFile(e.target.files[0])
      }
    };
    img.src = objectUrl;
  }
  const chooseFileUpload = () => document.getElementById('uploadImage').click()
  return (
    [<div style={{ cursor: 'pointer' }}>
      <input
        hidden
        id="uploadImage"
        type="file"
        accept="image/x-png,image/gif,image/jpeg"
        onChange={chooseImage}
        disabled={props.disabled}
        />
      <img
        src={props.image || imgPreview || 'https://www.seekpng.com/png/detail/110-1100707_person-avatar-placeholder.png'}
        alt="Profile"
        onClick={chooseFileUpload}
        width="100%"
        style={{maxHeight: '250px', maxWidth: '300px'}}
      />
      <div>
        <span style={{
          position: 'relative',
          left: 'calc(50% - 95px)',
          top: '-150px',
          fontSize: '2rem',
          color: '#adadad',
          display: 'block',
        }}
        onClick={chooseFileUpload}
        hidden={!!props.image || !!imgPreview}
        >Profile Image</span>
      </div>
    </div>,
    <CInputGroup className="">
      {!!error ? <CInput hidden invalid={!!error} /> : ""}
      {!!error ? <CInvalidFeedback>{error}</CInvalidFeedback> : ""}
    </CInputGroup>
    ]
  )
}

export default ContractForm