import React, { lazy, useState, useEffect } from 'react'
import { useHistory } from 'react-router-dom'
import {
  CBadge,
  CButton,
  CButtonGroup,
  CCard,
  CCardBody,
  CCardFooter,
  CCardHeader,
  CCol,
  CProgress,
  CRow,
  CCallout,
  CPagination
} from '@coreui/react'
import moment from 'moment'
import CIcon from '@coreui/icons-react'

import MainChartExample from '../charts/MainChartExample.js'
import { ContentTypes } from '../../constants'
import { dashboardApi } from '../../api'

const WidgetsDropdown = lazy(() => import('../widgets/WidgetsDropdown.js'))
const WidgetsBrand = lazy(() => import('../widgets/WidgetsBrand.js'))


const Dashboard = () => {
  const [topContents, setTopContents] = useState([])
  const [topProviders, setTopProviders] = useState([])
  const [totalView, setTotalView] = useState(100)
  const [expireContents, setExpireContents] = useState([])
  const [count, setCount] = useState(10)
  const limit = 10
  const [totalProviderView, setTotalProviderView] = useState(100)
  const [page, setPage] = useState(1)
  const history = useHistory()

  useEffect(() => {
    const fetchTopContents = async () => {
      const res = await dashboardApi.getTopContents()
      if (!res.isError) {
        setTopContents(res.data.top)
        setTotalView(res.data.top.reduce((s, t) => s + +t.count, 0))
      }
    }
    const fetchTopProviders = async () => {
      const res = await dashboardApi.getTopProviders()
      if (!res.isError) {
        setTopProviders(res.data.top)
        setTotalProviderView(res.data.top.reduce((s, t) => s + +t.count, 0))
      }
    }

    fetchTopContents()
    fetchTopProviders()
    fetchExpireContents(1, true)
  }, [])

  const pageChange = newPage => {
    if ((newPage || 1) === page) return
    fetchExpireContents(newPage)
  }

  const fetchExpireContents = async (page, fetchCount) => {
    let res = await dashboardApi.getExpireContents({limit, page, fetchCount})
    if (!res.isError) {
      setPage(page)
      let {contents, count} = res.data
      setExpireContents(contents)
      if (fetchCount) setCount(count)
    }
  }

  const viewContents = (id, contentType) => {
    switch (contentType.toLowerCase()) {
      case ContentTypes.album: {
        return history.push(`/albums/${id}`)
      }
      case ContentTypes.media: {
        return history.push(`/media/${id}`)
      }
      default: return
    }
  }

  const viewProvider = (id) => {
    history.push(`/providers/${id}`)
  }

  return (
    <>
      <WidgetsDropdown />
      <CCard>
        <CCardBody>
          <CRow>
            <CCol sm="5">
              <h4 id="traffic" className="card-title mb-0">Views</h4>
              {/* <div className="small text-muted">{}</div> */}
            </CCol>
            <CCol sm="7" className="d-none d-md-block">
              <CButton color="primary" className="float-right">
                <CIcon name="cil-cloud-download"/>
              </CButton>
              <CButtonGroup className="float-right mr-3">
                {
                  ['Day', 'Week', 'Month'].map(value => (
                    <CButton
                      color="outline-secondary"
                      key={value}
                      className="mx-0"
                      active={value === 'Week'}
                    >
                      {value}
                    </CButton>
                  ))
                }
              </CButtonGroup>
            </CCol>
          </CRow>
          <MainChartExample style={{height: '300px', marginTop: '40px'}}/>
        </CCardBody>
      </CCard>
      <CCard>
        <CCardBody>
          <CRow className="justify-content-center">
            <CCol sm="5" className="text-center mb-4">
              <h4 id="traffic" className="card-title mb-0">Top Contents</h4>
            </CCol>
          </CRow>
          <table className="table table-hover table-outline mb-0 d-none d-sm-table">
            <thead className="thead-light">
              <tr>
                <th className="text-center"><i className="fas fa-compact-disc" /></th>
                <th>Name</th>
                <th>Artist</th>
                <th>Content Type</th>
                <th>Views</th>
              </tr>
            </thead>
            <tbody>
              {topContents.map((content, idx) => (
                <tr key={idx} className="cursor-pointer" onClick={() => viewContents(content.contentId, content.contentType)}>
                  <td className="text-center">
                    <div className="c-avatar">
                      <img src={content.thumbnailPath || content.filePath} className="c-avatar-img" alt="" />
                    </div>
                  </td>
                  <td>
                    <div>{content.name}</div>
                  </td>
                  <td>
                    <div>{content.artistName}</div>
                  </td>
                  <td>
                    <div>{content.contentType?.toUpperCase()}</div>
                  </td>
                  <td>
                    <div className="clearfix">
                      <div className="float-left">
                        <strong>{Math.ceil(content.count * 100  / totalView)}%</strong>
                      </div>
                      <div className="float-right">
                        <small className="text-muted">{content.count} views</small>
                      </div>
                    </div>
                    <CProgress className="progress-xs" color="success" value={Math.ceil(content.count * 100 / totalView)} />
                  </td>
                </tr>
              ))}
            </tbody>
          </table>
        </CCardBody>
      </CCard>
      { !!topProviders?.length &&
        <CCard>
          <CCardBody>
            <CRow className="justify-content-center">
              <CCol sm="5" className="text-center mb-4">
                <h4 id="traffic" className="card-title mb-0">Top Providers</h4>
              </CCol>
            </CRow>
            <table className="table table-hover table-outline mb-0 d-none d-sm-table">
              <thead className="thead-light">
                <tr>
                  <th className="text-center"><i className="fas fa-user-tie" /></th>
                  <th>Name</th>
                  <th>Content Upload</th>
                  <th className="text-center">Created At</th>
                </tr>
              </thead>
              <tbody>
                {topProviders.map((content, idx) => (
                  <tr key={idx}  className="cursor-pointer" onClick={() => viewProvider(content.providerId)}>
                    <td className="text-center">
                      <div className="c-avatar">
                        <img src={content.thumbnailPath || content.filePath} className="c-avatar-img" alt="" />
                      </div>
                    </td>
                    <td>
                      <div>{content.name}</div>
                    </td>
                    <td>
                      <div className="clearfix">
                        <div className="float-left">
                          <strong>{Math.ceil(content.count * 100  / totalProviderView)}%</strong>
                        </div>
                        <div className="float-right">
                          <small className="text-muted">{content.count} contents</small>
                        </div>
                      </div>
                      <CProgress className="progress-xs" color="success" value={Math.ceil(content.count * 100 / totalProviderView)} />
                    </td>
                    <td className="text-center">
                      {moment(content.createdAt).format('DD-MM-YYYY')}
                    </td>
                  </tr>
                ))}
              </tbody>
            </table>
          </CCardBody>
        </CCard>
      }
      <CCard>
        <CCardBody>
          <CRow className="justify-content-center">
            <CCol sm="5" className="text-center mb-4">
              <h4 id="traffic" className="card-title mb-0 text-warning">Warning Expire Contents</h4>
            </CCol>
          </CRow>
          <table className="table table-hover table-outline mb-0 d-none d-sm-table">
            <thead className="thead-light">
              <tr>
                <th className="text-center"><i className="fas fa-compact-disc" /></th>
                <th>Name</th>
                <th>Artist</th>
                <th>Content Type</th>
                <th>Contract Date</th>
              </tr>
            </thead>
            <tbody>
              {expireContents.map((content, idx) => (
                <tr key={idx} className="cursor-pointer" onClick={() => viewContents(content.contentId, content.contentType)}>
                  <td className="text-center">
                    <div className="c-avatar">
                      <img src={content.thumbnailPath || content.filePath} className="c-avatar-img" alt="" />
                    </div>
                  </td>
                  <td>
                    <div>{content.name}</div>
                  </td>
                  <td>
                    <div>{content.artistName}</div>
                  </td>
                  <td>
                    <div>{content.contentType?.toUpperCase()}</div>
                  </td>
                  <td>
                    <div>{moment(content.contractDate).format('DD-MM-YYYY')}</div>
                  </td>
                </tr>
              ))}
            </tbody>
          </table>
          <CPagination
            activePage={page}
            onActivePageChange={pageChange}
            pages={Math.ceil(count / limit)}
            doubleArrows={false}
            align="center"
          />
        </CCardBody>
      </CCard>
    </>
  )
}

export default Dashboard
