import React, { useEffect, useState } from 'react'
import { useHistory } from 'react-router-dom'
import {
  CCard,
  CCardBody,
  CCardHeader,
  CFormGroup,
  CInputGroup,
  CInputGroupPrepend,
  CInputGroupText,
  CInput,
  CInputGroupAppend,
  CInvalidFeedback,
  CCardFooter,
  CButton,
  CSelect,
  CForm,
  CTextarea,
  CTooltip,
} from '@coreui/react'
import { mediaApi, artistsApi, providersApi, authApi, genresApi, configsApi, contractsApi } from '../../../api'
import { ACTIONS, MODELS, businessTypes } from '../../../constants'
import UploadMediaImages from './Images'
import MultipleSelectDropdown from '../../../containers/MultipleSelectDropdown'
import GroupSelect from '../../../containers/GroupSelect'
import AutocompleteInput from '../../../containers/AutocompleteInput'

const MediaForm = (props) => {
  const { match: {params: { id }}} = props
  const history = useHistory()
  const [newMedium, setNewMedium] = useState({})
  // const [artists, setArtists] = useState({artists: [], authors: [], composers: []})
  const [artists, setArtists] = useState({artists: []})
  const [providers, setProviders] = useState([])
  const [contracts, setContracts] = useState([])
  const [genres, setGenres] = useState({})
  const [permissions, setPermissions] = useState({
    editableMedium: false,
    editableProvider: false
  })
  const [errors, setErrors] = useState({})
  const [covers, setCovers] = useState([])
  const [banners, setBanners] = useState([])
  const [isLoading, setLoading] = useState(false)
  const [files, setFiles] = useState({
    fileFull: null,
    fileIntro: null,
    fileMv: null,
    fileBeat: null,
    covers: [],
    banners: []
  })
  const [originalFiles, setOriginalFiles] = useState({
    fileFull: "",
    fileIntro: "",
    fileMv: "",
    fileBeat: "",
  })
  const [configs, setConfigs] = useState({})
  // const [expands, setExpands] = useState({
  //   images: true,
  //   artist: true,
  //   files: true,
  // })

  useEffect(() => {
    const fetchConfigs = async () => {
      const res = await configsApi.getConfigs()
      if (!res.isError) {
        setConfigs(res.data.configs)
      }
    }
    const fetchPermissions = async () => {
      const editableProvider = await authApi.checkPermission(MODELS.PROVIDER_DETAILS, ACTIONS.EDIT)
      const editableMedium = await authApi.checkPermission(MODELS.MEDIUM_DETAILS, ACTIONS.EDIT)

      setPermissions({ editableMedium, editableProvider })
    }
    fetchConfigs()
    fetchPermissions()
  }, [])

  useEffect(() => {
    const fetchMedia = async (id) => {
      setLoading(true)
      const res = await mediaApi.getOne(id)
      if (res.isError || !res?.data?.medium) {
        setErrors({general: res.errorMessage || 'Medium not found'})
        setTimeout(() => history.push('/media'), 3000)
        return
      }
      const resMedium = res.data.medium
      const covers = resMedium.images.map(p => p?.filePath)
      const banners = resMedium.banner?.filePath ? [resMedium.banner?.filePath] : []
      const fileFull = resMedium.fileFull?.name
      const fileIntro = resMedium.fileIntro?.name
      const fileBeat = resMedium.fileBeat?.name
      const fileMv = resMedium.fileMv?.name
      delete resMedium.fileFull
      delete resMedium.fileIntro
      delete resMedium.fileBeat
      delete resMedium.fileMv
      resMedium.artists = resMedium.artists.map(a => a.artistId)
      resMedium.composerName = resMedium.composers?.[0]?.name
      resMedium.authorName = resMedium.authors?.[0]?.name
      resMedium.genre = resMedium.genres?.[0] ? {label: resMedium.genres[0].category, value: resMedium.genres[0].id} : null
      resMedium.contractId = resMedium.contract ? {label: resMedium.contract.id, value: resMedium.contract.id} : null
      resMedium.artistName = resMedium.mainArtist?.name || ''
      resMedium.contractStartDate = resMedium.contract?.startDate?.slice(0, 10)
      resMedium.contractEndDate = resMedium.contract?.endDate?.slice(0, 10)

      setNewMedium({
        ...resMedium,
      })
      setCovers(covers)
      setBanners(banners)
      setFiles({
        ...files,
        covers,
        banners,
      })
      setOriginalFiles({
        fileFull, fileIntro, fileBeat, fileMv
      })
      setLoading(false)
    }

    const formatGenresByGroup = (genres) => {
      const genreGroups = {}
      const parents = {}
      genres.forEach(g => {
        if (g.level === 0) {
          parents[g.id] = g
          genreGroups[g.category] = []
          return
        }
      })
      genres.forEach((g) => {
        if (g.level === 0) return
        let parentCategory = parents[g.parentId]?.category
        if (!parentCategory) return
        genreGroups[parentCategory]?.push(g)
      })
      return genreGroups
    }

    const fetchGenres = async () => {
      const res = await genresApi.getListForSelect()
      if (!res.isError) {
        const { genres } = res.data
        return setGenres(formatGenresByGroup(genres))
      }
    }

    fetchArtirts()
    fetchGenres()
    if (id) fetchMedia(id)
    else {
      let userInfo = localStorage.getItem('userProfile')
      try {
        userInfo = JSON.parse(userInfo)
        setNewMedium({...newMedium, providerId: userInfo.providerId, initiatorName: userInfo.name})
      }
      catch (err) {
        console.log({ERROR: err.message})
      }
    }
  }, [id, history])

  useEffect(() => {
    const fetchProviders = async () => {
      const res = await providersApi.getListForSelect()
      if (!res.isError && res?.data?.providers) {
        setProviders(res.data.providers)
      }
    }
    const fetchContracts = async () => {
      const res = await contractsApi.getListForSelect()
      if (!res.isError && res?.data?.contracts) {
        setContracts(res.data.contracts)
      }
    }

    fetchProviders()
    fetchContracts()
  }, [])

  const fetchArtirts = async () => {
    let res = await artistsApi.fetchArtistsAndGroup()
    if (!res.isError) {
      setArtists(res.data)
    }
  }

  const onBack = () => {
    history.goBack()
  }

  const onChangeValue = ({target: { name, value }}) => {
    setErrors({...errors, [name]: '', general: ''})
    const updateMedium = {
      [name]: value
    }
    if (name === 'contractId') {
      value = value.value
      let selectedContract = contracts.find(c => c.id === value)
      const contractStart = selectedContract?.startDate?.slice(0, 10)
      const contractEnd = selectedContract?.endDate?.slice(0, 10)
      updateMedium.contractStartDate = contractStart
      updateMedium.contractEndDate = contractEnd
    }
    setNewMedium({...newMedium, ...updateMedium})
  }

  const onSelectArtist = (name,  values ) => {
    setNewMedium({...newMedium, [name]: values})
  }

  const openUpload = (id) => {
    document.getElementById(id).click()
  }

  const uploadFile = ({ target: { name, files: inputFiles }}) => {
    if (inputFiles && inputFiles[0]) {
      const fileType = inputFiles[0].type.split('/')[0]
      const size = inputFiles[0].size
      let sizeConfigKey  = `${name}MaxSize`
      const isOverSize = configs[sizeConfigKey] ? size > configs[sizeConfigKey] * 1024 * 1024 : false
      if (isOverSize) {
        return setErrors({
          ...errors,
          [name]: `File size is over ${configs[sizeConfigKey]} MB`
        })
      }
      setErrors({...errors, [name]: ""})
      setNewMedium({...newMedium, [name]: inputFiles[0].name})
      setFiles({...files, [name]: inputFiles[0]})
    }
  }

  const onUploadImages = (type, imageUrls, inputFiles) => {
    if (type === 'covers') setCovers([...covers, ...imageUrls])
    else setBanners([...banners, ...imageUrls])
    setFiles({...files, [type]: [...files[type], ...inputFiles]})
  }

  const onRemoveImage = (type, pos) => {
    if (type === 'covers') setCovers(covers.filter((_, i) => i !== pos))
    else setBanners(banners.filter((_, i) => i !== pos))
    setFiles({...files, [type]: files[type].filter((_, i) => i !== pos)})
  }

  const removeFile = (e, fileField) => {
    e.stopPropagation()
    setFiles({
      ...files,
      [fileField]: null
    })
    setNewMedium({
      ...newMedium,
      [fileField]: ""
    })
    document.getElementById(fileField).value = null
  }

  const generateRelationFiles = (media, files) => {
    const isNotUpdateImageIds = files.covers.map((file, idx) => typeof file === 'string' ? media.images[idx].id : "").filter(id => !!id)
    const submitMedia = {...media, isNotUpdateImageIds}
    if (files.fileFull) submitMedia.fileFull = files.fileFull
    if (files.fileIntro) submitMedia.fileIntro = files.fileIntro
    if (files.fileBeat) submitMedia.fileBeat = files.fileBeat
    if (files.fileMv) submitMedia.fileMv = files.fileMv
    if (files.banners?.length > 0) submitMedia.banner = files.banners[0]
    submitMedia.images = files.covers.filter(f => typeof f !== 'string')
    submitMedia.genres = submitMedia.genre ? [submitMedia.genre?.value] : []
    submitMedia.contractId = submitMedia?.contractId.value || null
    return submitMedia
  }

  const onSubmit = async () => {
    let validateErrors = {};
    if (!newMedium.name) {
      validateErrors['name'] = 'Media name is required!';
    }
    if (newMedium.artistName) {
      const artist = artists?.artists?.find(a => a.name.trim() === newMedium.artistName.trim())
      if (!artist) {
        validateErrors['artistName'] = 'Please select main artist in dropdown!'
      } else {
        newMedium.artistId = artist.id
      }
    } else {
      delete newMedium.artistId
    }
    if (newMedium.authorName) {
      const artist = artists?.artists?.find(a => a.name.trim() === newMedium.authorName.trim())
      if (!artist) {
        validateErrors['authorName'] = 'Please select author in dropdown!'
      } else {
        newMedium.authors = [artist.id]
      }
    } else {
      delete newMedium.authors
    }
    if (newMedium.composerName) {
      const artist = artists?.artists?.find(a => a.name.trim() === newMedium.composerName.trim())
      if (!artist) {
        validateErrors['composerName'] = 'Please select author in dropdown!'
      } else {
        newMedium.composers = [artist.id]
      }
    } else {
      delete newMedium.composers
    }
    if (newMedium.contractId?.value) {
      const contract = contracts?.find(a => a.id === newMedium.contractId?.value.trim())
      if (!contract) {
        validateErrors['contractId'] = 'Please select a contract in suggestion list!'
      }
    } else {
      delete newMedium.contractId
    }
    if (Object.values(validateErrors).length) return setErrors(validateErrors)
    setLoading(true)
    if (id) {
      const editMedia = generateRelationFiles(newMedium, files)
      let res = await mediaApi.edit(id, editMedia)

      if (res.isError) {
        setLoading(false)
        return setErrors({general: res.errorMessage})
      }

      return props.history.push('/media')
    }

    let res = await mediaApi.create({
      ...generateRelationFiles(newMedium, files),
      status: 'active',
    })

    if (res.isError) {
      setLoading(false)
      return setErrors({general: res.errorMessage})
    }
    props.history.push('/media')
  }

  return (
    <CForm className="form-horizontal" formAction="">
      <CCard className="mb-2">
        <CCardHeader align="center">
          <h1>{id ? 'Edit Media' : 'Register Media'}</h1>
        </CCardHeader>
        <CCardBody className="row justify-content-center">
          <div className="col-5">
            <UploadMediaImages
              disabled={!permissions.editableMedium || isLoading}
              covers={covers}
              banners={banners}
              onUploadCovers={(...args) => onUploadImages('covers', ...args)}
              onRemoveCover={(...args) => onRemoveImage('covers', ...args)}
              onUploadBanners={(...args) => onUploadImages('banners', ...args)}
              onRemoveBanner={(...args) => onRemoveImage('banners', ...args)}
              configs={configs}
            />
          </div>
          <div className="col-7">
            <CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CInputGroupText style={{ width: "150px"}} >Media Name</CInputGroupText>
                </CInputGroupPrepend>
                <CInput
                  onChange={onChangeValue}
                  defaultValue={newMedium ? newMedium.name : ''} type="text"
                  id="name"
                  name="name"
                  invalid={!!errors["name"]}
                  disabled={!permissions.editableMedium || isLoading}
                />
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
                {!!errors["name"] ? <CInvalidFeedback>{errors["name"]}</CInvalidFeedback> : ""}
              </CInputGroup>
            </CFormGroup>
            <CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CInputGroupText style={{ width: "150px"}} >Provider</CInputGroupText>
                </CInputGroupPrepend>
                  <CSelect
                    onChange={onChangeValue}
                    value={newMedium ? newMedium.providerId : ''}
                    disabled={!permissions.editableMedium || !permissions.editableProvider || isLoading}
                    type="text"
                    id="providerId"
                    name="providerId"
                    invalid={!!errors["providerId"]}
                  >
                    <option value="">Select Provider</option>
                    {providers.map((provider, pos) => (
                      <option value={provider.id} key={pos}>{provider.name}</option>
                    ))}
                  </CSelect>
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
                {!!errors["providerId"] ? <CInvalidFeedback>{errors["providerId"]}</CInvalidFeedback> : ""}
              </CInputGroup>
            </CFormGroup>
            <CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CInputGroupText style={{ width: "150px"}} >Genres</CInputGroupText>
                </CInputGroupPrepend>
                <GroupSelect
                  className="multiple-select"
                  name="genre"
                  optionGroups={genres}
                  textField="category"
                  valueField="id"
                  value={newMedium.genre}
                  onChange={onChangeValue}
                />
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
                {!!errors["genre"] ? <CInvalidFeedback>{errors["genre"]}</CInvalidFeedback> : ""}
              </CInputGroup>
            </CFormGroup>
            <CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CInputGroupText style={{ width: "150px"}} >Type</CInputGroupText>
                </CInputGroupPrepend>
                  <CSelect
                    onChange={onChangeValue}
                    value={newMedium ? newMedium.type : ''}
                    disabled={!permissions.editableMedium || isLoading}
                    type="text"
                    id="type"
                    name="type"
                    invalid={!!errors["type"]}
                  >
                    <option value="">Select Type</option>
                    {businessTypes.map((type, pos) => (
                      <option value={type.value} key={pos}>{type.name}</option>
                    ))}
                  </CSelect>
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
                {!!errors["type"] ? <CInvalidFeedback>{errors["type"]}</CInvalidFeedback> : ""}
              </CInputGroup>
            </CFormGroup>
            <CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CInputGroupText style={{ width: "150px"}} >Initiator</CInputGroupText>
                </CInputGroupPrepend>
                <CInput
                  onChange={onChangeValue}
                  defaultValue={newMedium ? newMedium.initiatorName : ''}
                  type="text"
                  id="initiatorName"
                  name="initiatorName"
                  className="no-icon"
                  readOnly
                />
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
              </CInputGroup>
            </CFormGroup>
            <CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CInputGroupText style={{ width: "150px"}} >Contract Number</CInputGroupText>
                </CInputGroupPrepend>
                <GroupSelect
                  className="multiple-select"
                  name="contractId"
                  optionGroups={contracts}
                  textField="id"
                  valueField="id"
                  value={newMedium.contractId}
                  onChange={onChangeValue}
                />
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
              </CInputGroup>
            </CFormGroup>
            <CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CInputGroupText style={{ width: "150px"}} >Contract Start Date</CInputGroupText>
                </CInputGroupPrepend>
                <CInput
                  onChange={onChangeValue}
                  defaultValue={newMedium ? newMedium.contractStartDate : ''}
                  type="text"
                  id="contractStartDate"
                  name="contractStartDate"
                  className="no-icon"
                  readOnly
                />
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
              </CInputGroup>
            </CFormGroup>
            <CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CInputGroupText style={{ width: "150px"}} >Contract End Date</CInputGroupText>
                </CInputGroupPrepend>
                <CInput
                  onChange={onChangeValue}
                  defaultValue={newMedium ? newMedium.contractEndDate : ''}
                  type="text"
                  id="contractEndDate"
                  name="contractEndDate"
                  className="no-icon"
                  readOnly
                />
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
              </CInputGroup>
            </CFormGroup>
            <CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CInputGroupText style={{ width: "150px"}} >ISRC Code</CInputGroupText>
                </CInputGroupPrepend>
                <CInput
                  onChange={onChangeValue}
                  defaultValue={newMedium ? newMedium.isrc : ''} type="text"
                  id="isrc"
                  name="isrc"
                  className="no-icon"
                  disabled={!permissions.editableMedium || isLoading}
                />
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-hashtag" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
              </CInputGroup>
            </CFormGroup>
            <CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CInputGroupText style={{ width: "150px"}} >Singers</CInputGroupText>
                </CInputGroupPrepend>
                <MultipleSelectDropdown
                  options={artists.artists}
                  values={newMedium.artists || []}
                  disabled={!permissions.editableMedium || isLoading}
                  valueKey="id"
                  textKey="name"
                  name="artistSelect"
                  onSelect={(values) => onSelectArtist('artists', values)}
                  onRemove={() => onSelectArtist('artists', [])}
                />
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
                {!!errors["artistName"] ? <CInvalidFeedback>{errors["artistName"]}</CInvalidFeedback> : ""}
              </CInputGroup>
            </CFormGroup>
            <AutocompleteInput
              className="col-12 row"
              prependInput={
                <CInputGroupPrepend>
                  <CInputGroupText style={{ width: "150px"}} >Author</CInputGroupText>
                </CInputGroupPrepend>
              }
              appendInput={
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
              }
              error={errors["authorName"]}
              name="authorName"
              onChange={onChangeValue}
              suggestions={artists?.artists?.filter(a => a.professions.includes('Author'))?.map(a => a.name)}
              value={newMedium.authorName}
              disabled={permissions.editableMedium || isLoading}
            />
            <AutocompleteInput
              className="col-12 row"
              prependInput={
                <CInputGroupPrepend>
                  <CInputGroupText style={{ width: "150px"}} >Producer</CInputGroupText>
                </CInputGroupPrepend>
              }
              appendInput={
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
              }
              error={errors["composerName"]}
              name="composerName"
              onChange={onChangeValue}
              suggestions={artists?.artists?.filter(a => a.professions.includes('Composer'))?.map(a => a.name)}
              value={newMedium.composerName}
              disabled={permissions.editableMedium || isLoading}
            />
            <AutocompleteInput
              className="col-12 row"
              prependInput={
                <CInputGroupPrepend>
                  <CInputGroupText style={{ width: "150px"}} >Main Artist</CInputGroupText>
                </CInputGroupPrepend>
              }
              appendInput={
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
              }
              error={errors["artistName"]}
              name="artistName"
              onChange={onChangeValue}
              suggestions={artists?.artists?.map(a => a.name)}
              value={newMedium.artistName}
              disabled={permissions.editableMedium || isLoading}
            /><CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CTooltip content={`Audio/Video max ${configs.fileFullMaxSize} MB`} placement="top-start">
                    <CInputGroupText style={{ width: "150px"}}>File Full <i className="far fa-question-circle pl-2"/></CInputGroupText>
                  </CTooltip>
                </CInputGroupPrepend>
                <div className={"form-control file-input" + (errors["fileFull"] ? ' is-invalid' : '')} onClick={() => openUpload('fileFull')} style={{ cursor: 'pointer' }}>
                  {newMedium?.fileFull || originalFiles?.fileFull || 'Choose File...'}
                  {!!files.fileFull &&
                    <CButton className="btn-remove-file" onClick={(e) => removeFile(e, 'fileFull')}>
                      <svg width="24" height="24" fill="none" stroke="currentColor" stroke-width="2" class="dropdown-search-clear-icon gray"><line x1="18" y1="6" x2="6" y2="18"></line><line x1="6" y1="6" x2="18" y2="18"></line></svg>
                    </CButton>
                  }
                </div>
                <input disabled={!permissions.editableMedium || isLoading} type="file" accept="video/*,audio/*" id="fileFull" name="fileFull" hidden onChange={uploadFile}/>
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-file-audio" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
                {!!errors["fileFull"] ? <CInvalidFeedback>{errors["fileFull"]}</CInvalidFeedback> : ""}
              </CInputGroup>
            </CFormGroup>
            <CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CTooltip content={`Audio/Video max ${configs.fileIntroMaxSize} MB`} placement="top-start">
                    <CInputGroupText style={{ width: "150px"}}>File 15/30s <i className="far fa-question-circle pl-2"/></CInputGroupText>
                  </CTooltip>
                </CInputGroupPrepend>
                <div className={"form-control file-input" + (errors["fileIntro"] ? ' is-invalid' : '')} onClick={() => openUpload('fileIntro')} style={{ cursor: 'pointer' }}>
                  {newMedium?.fileIntro || originalFiles?.fileIntro || 'Choose File...'}
                  {!!files.fileIntro &&
                    <CButton className="btn-remove-file" onClick={(e) => removeFile(e, 'fileIntro')}>
                      <svg width="24" height="24" fill="none" stroke="currentColor" stroke-width="2" class="dropdown-search-clear-icon gray"><line x1="18" y1="6" x2="6" y2="18"></line><line x1="6" y1="6" x2="18" y2="18"></line></svg>
                    </CButton>
                  }
                </div>
                <input disabled={!permissions.editableMedium || isLoading} type="file" accept="video/*,audio/*" id="fileIntro" name="fileIntro" hidden onChange={uploadFile}/>
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-file-audio" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
                {!!errors["fileIntro"] ? <CInvalidFeedback>{errors["fileIntro"]}</CInvalidFeedback> : ""}
              </CInputGroup>
            </CFormGroup>
            <CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CTooltip content={`Video max ${configs.fileMvMaxSize} MB`} placement="top-start">
                    <CInputGroupText style={{ width: "150px"}}>File MV <i className="far fa-question-circle pl-2"/></CInputGroupText>
                  </CTooltip>
                </CInputGroupPrepend>
                <div className={"form-control file-input" + (errors["fileMv"] ? ' is-invalid' : '')} onClick={() => openUpload('fileMv')} style={{ cursor: 'pointer' }}>
                  {newMedium?.fileMv || originalFiles?.fileMv || 'Choose File...'}
                  {!!files.fileMv &&
                    <CButton className="btn-remove-file" onClick={(e) => removeFile(e, 'fileMv')}>
                      <svg width="24" height="24" fill="none" stroke="currentColor" stroke-width="2" class="dropdown-search-clear-icon gray"><line x1="18" y1="6" x2="6" y2="18"></line><line x1="6" y1="6" x2="18" y2="18"></line></svg>
                    </CButton>
                  }
                </div>
                <input disabled={!permissions.editableMedium || isLoading} type="file" accept="video/*" id="fileMv" name="fileMv" hidden onChange={uploadFile}/>
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-file-audio" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
                {!!errors["fileMv"] ? <CInvalidFeedback>{errors["fileMv"]}</CInvalidFeedback> : ""}
              </CInputGroup>
            </CFormGroup>
            <CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CTooltip content={`Audio max ${configs.fileBeatMaxSize} MB`} placement="top-start">
                    <CInputGroupText style={{ width: "150px"}}>File Beat <i className="far fa-question-circle pl-2"/></CInputGroupText>
                  </CTooltip>
                </CInputGroupPrepend>
                <div className={"form-control file-input" + (errors["fileBeat"] ? ' is-invalid' : '')} onClick={() => openUpload('fileBeat')} style={{ cursor: 'pointer' }}>
                  {newMedium?.fileBeat || originalFiles?.fileBeat || 'Choose File...'}
                  {!!files.fileBeat &&
                    <CButton className="btn-remove-file" onClick={(e) => removeFile(e, 'fileBeat')}>
                      <svg width="24" height="24" fill="none" stroke="currentColor" stroke-width="2" class="dropdown-search-clear-icon gray"><line x1="18" y1="6" x2="6" y2="18"></line><line x1="6" y1="6" x2="18" y2="18"></line></svg>
                    </CButton>
                  }
                </div>
                <input disabled={!permissions.editableMedium || isLoading} type="file" accept="audio/*" id="fileBeat" name="fileBeat" hidden onChange={uploadFile}/>
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-file-audio" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
                {!!errors["fileBeat"] ? <CInvalidFeedback>{errors["fileBeat"]}</CInvalidFeedback> : ""}
              </CInputGroup>
            </CFormGroup>
            <CFormGroup className="col-12 row">
              <CInputGroup>
                <CInputGroupPrepend>
                  <CInputGroupText style={{ width: "150px"}} >Lyric</CInputGroupText>
                </CInputGroupPrepend>
                <CTextarea
                  onChange={onChangeValue}
                  value={newMedium ? newMedium.lyric : ''}
                  disabled={!permissions.editableMedium || isLoading}
                  type="text"
                  id="lyric"
                  name="lyric"
                  invalid={!!errors["lyric"]}
                />
                <CInputGroupAppend>
                  <CInputGroupText><i className="fas fa-book-open" style={{ width: '20px' }}/></CInputGroupText>
                </CInputGroupAppend>
                {!!errors["lyric"] ? <CInvalidFeedback>{errors["lyric"]}</CInvalidFeedback> : ""}
              </CInputGroup>
            </CFormGroup>
            <CFormGroup className="col-12 row">
              <CInputGroup className="mb-2">
                {!!errors["general"] ? <CInput hidden invalid={!!errors["general"]} /> : ""}
                {!!errors["general"] ? <CInvalidFeedback>{errors["general"]}</CInvalidFeedback> : ""}
              </CInputGroup>
            </CFormGroup>
          </div>
        </CCardBody>
        <CCardFooter className="row justify-content-around pt-3 pb-0">
          <div className="col-6 mb-3 row justify-content-around">
            <CButton
              disabled={!permissions.editableMedium || isLoading}
              type="button"
              className="btn-pill col-4"
              size="lg"
              color="primary"
              onClick={onSubmit}
            >
              {!!isLoading && <span class="spinner-border spinner-border-sm" role="status" aria-hidden="true"></span>}
              {isLoading ? "Submitting..." : "Submit"}
            </CButton>
            <CButton type="button" disabled={isLoading} className="btn-pill col-4" size="lg" color="dark" onClick={onBack}>Back</CButton>
          </div>
        </CCardFooter>
      </CCard>
      {/* <CCard className="mb-2">
        <CCardHeader className="py-1 cursor-pointer" onClick={() => collapsePanel('artist')}>
          <h3>Artists <i className={"pt-1 fas fa-chevron-" + (expands.artist ? "up" : "down")}></i></h3>
        </CCardHeader>
        <CCardBody className={expands.artist ? "" : " d-none"}>
          <div className="col-12 row justify-content-center align-items-center">
          </div>
        </CCardBody>
      </CCard> */}

      {/* <CCard className="mb-2">
        <CCardHeader className="py-1 cursor-pointer" onClick={() => collapsePanel('images')}>
          <h3>Images <i className={"pt-1 fas fa-chevron-" + (expands.images ? "up" : "down")}></i></h3>
        </CCardHeader>
        <CCardBody className={expands.images ? "" : " d-none"}>
        </CCardBody>
      </CCard> */}
      {/* <CCard className="mb-2">
        <CCardHeader className="py-1 cursor-pointer" onClick={() => collapsePanel('files')}>
          <h3>Files <i className={"pt-1 fas fa-chevron-" + (expands.files ? "up" : "down")}></i></h3>
        </CCardHeader>
        <CCardBody className={expands.files ? "" : " d-none"}>
          <div className="col-12 row justify-content-center align-items-center">

          </div>
        </CCardBody>
      </CCard> */}
      {/* <CCard>
      </CCard> */}
    </CForm>
  )
}

export default MediaForm