import React, { useEffect, useState } from 'react'
import {
  CCard,
  CCardBody,
  CCardHeader,
  CTabs,
  CNav,
  CNavLink,
  CNavItem,
  CButton,
  CDataTable,
} from '@coreui/react'

import { permissionsApi } from '../../api'
import { MODELS, roles } from '../../constants'

const fields = ['model', 'Tick All', 'view', 'edit', 'delete', 'Edited By'];
const ArtistForm = (props) => {
  const [activeTab, setActiveTab] = useState(0)
  const [isLoading, setLoading] = useState(false)
  const [errors, setErrors] = useState({})
  const [permissions, setPermissions] = useState({})
  const [selectedRole, setSelectedRole] = useState('SUPER_ADMIN')

  useEffect(() => {
    if (!selectedRole) return

    fetchPermissions({role: selectedRole})
  }, [selectedRole])

  const fetchPermissions = async (filter) => {
    const res = await permissionsApi.getList(filter)
    if (!res.isError) {
      setPermissions(res.data.permissions)
    }
  }

  const onChangeValue = ({ target: { name, value, checked }}) => {
    let permission = {...(permissions[name] || {permission: []})}
    if (!checked) {
      permission.permission = permission.permission.filter(p => p !== value)
    } else {
      permission.permission.push(value)
    }

    setPermissions({
      ...permissions,
      [name]: permission
    })
  }

  const onTickAll = ({ target: { name, checked }}) => {
    let permission = {...(permissions[name] || {permission: []})}
    if (!checked) {
      permission.permission = []
    } else {
      permission.permission = ['VIEW', 'WRITE', 'DELETE']
    }

    setPermissions({
      ...permissions,
      [name]: permission
    })
  }

  const onTabChange = (e) => {
    setActiveTab(e)
    setSelectedRole(roles[e].value)
    setErrors({...errors, selectedRole: ''})
  }

  const onSubmit = async () => {
    setLoading(true)
    let res = await permissionsApi.editAllPermissions({role: selectedRole, permissions});
    setLoading(false)
    if (!res.isError) {
      return fetchPermissions({role: selectedRole})
    }
    setErrors({...errors, selectedRole: res.errorMessage})
  }

  return (
    <CCard>
      <CCardHeader align="center">
        <h1>Permissions Editor</h1>
      </CCardHeader>

      <CTabs variant="pills" activeTab={activeTab} onActiveTabChange={onTabChange}>
        <CNav variant="tabs">
          {roles.map((role, idx) => (
            <CNavItem key={idx}>
              <CNavLink style={{fontWeight: 'bold'}}>
                {role.value.replace('_', ' ')}
              </CNavLink>
            </CNavItem>
          ))}
        </CNav>
        <CCardBody>
          <div className="row justify-content-center">
            <div className="row col-8">
              <CDataTable
                items={Object.keys(MODELS)}
                fields={fields}
                hover
                // onRowClick={onSelectRow}
                // clickableRows
                scopedSlots = {{
                  'model':
                    (item)=>(
                      <td style={{verticalAlign: "middle"}}>
                        {item}
                      </td>
                    ),
                  'Tick All':
                    (item)=>(
                      <td className="pl-4">
                        <input
                          className="cursor-pointer"
                          style={{width: "20px", height: "20px"}}
                          type="checkbox"
                          checked={
                            permissions[item]?.permission?.includes('VIEW') &&
                            permissions[item]?.permission?.includes('WRITE') &&
                            permissions[item]?.permission?.includes('DELETE')
                          }
                          onChange={onTickAll}
                          name={item}
                        />
                      </td>
                    ),
                  'view':
                    (item)=>(
                      <td className="pl-4">
                        <input
                          className="cursor-pointer"
                          style={{width: "20px", height: "20px"}}
                          type="checkbox"
                          checked={permissions[item]?.permission?.includes('VIEW') || false}
                          onChange={onChangeValue}
                          name={item}
                          value={'VIEW'}
                        />
                      </td>
                    ),
                  'edit':
                    (item)=>(
                      <td className="pl-4">
                        <input
                          className="cursor-pointer"
                          style={{width: "20px", height: "20px"}}
                          type="checkbox"
                          checked={permissions[item]?.permission?.includes('WRITE') || false}
                          onChange={onChangeValue}
                          name={item}
                          value={'WRITE'}
                        />
                      </td>
                    ),
                  'delete':
                    (item)=>(
                      <td className="pl-4">
                        <input
                          className="cursor-pointer"
                          style={{width: "20px", height: "20px"}}
                          type="checkbox"
                          checked={permissions[item]?.permission?.includes('DELETE') || false}
                          onChange={onChangeValue}
                          name={item}
                          value={'DELETE'}
                        />
                      </td>
                    ),
                  'Edited By':
                    (item)=>(
                      <td style={{verticalAlign: "middle"}}>
                        {permissions[item]?.editedUser?.email || ""}
                      </td>
                    ),
                }}
              />
            </div>
          </div>
          <div className="row justify-content-center">
            <div className="col-3">
              <CButton
                disabled={isLoading}
                type="button"
                className="btn-pill col-12"
                size="lg"
                color="primary"
                onClick={onSubmit}
              >
                {!!isLoading && <span class="spinner-border spinner-border-sm" role="status" aria-hidden="true"></span>}
                {isLoading ? "Saving..." : "Save"}
              </CButton>
            </div>
          </div>
        </CCardBody>
      </CTabs>
    </CCard>
  )
}

export default ArtistForm