import React, { useEffect, useState } from 'react'
import {
  CCard,
  CCardBody,
  CCardHeader,
  CFormGroup,
  CInputGroup,
  CInputGroupPrepend,
  CInputGroupText,
  CInput,
  CInputGroupAppend,
  CInvalidFeedback,
  CCardFooter,
  CButton,
  CForm,
} from '@coreui/react'
import { providersApi, authApi } from '../../../api'
import { MODELS, ACTIONS } from '../../../constants'

const ProviderForm = (props) => {
  const { match: {params: { id }}, history } = props

  const [newProvider, setNewProvider] = useState({})
  const [isLoading, setLoading] = useState(false)
  const [errors, setErrors] = useState({})
  const [image, setImage] = useState(null)
  const [imageFile, setImageFile] = useState(null)
  const [permissions, setPermissions] = useState({
    editableProvider: false
  })

  useEffect(() => {
    const fetchPermissions = async () => {
      const editableProvider = await authApi.checkPermission(MODELS.PROVIDER_DETAILS, ACTIONS.EDIT)

      setPermissions({ editableProvider })
    }

    fetchPermissions()
  }, [])

  useEffect(() => {
    const fetchProviderDetails = async () => {
      if (!id) return
      setLoading(true)
      const res = await providersApi.getOne(id)
      setLoading(false)
      if (res.isError || !res?.data?.provider) {
        setErrors({general: res.errorMessage || 'Provider not found'})
        setTimeout(() => history.push('/providers'), 3000)
        return
      }
      const resProvider = res.data.provider
      let image = resProvider.profileImage?.filePath
      const contractDate = resProvider.contractDate
      delete resProvider.profileImage
      setNewProvider({...resProvider, contractDate: contractDate.slice(0, 10)})
      setImage(image)
    }
    fetchProviderDetails()
  }, [id, history])
  const onBack = () => {
    props.history.goBack()
  }

  const onUploadImage = (img) => {
    setImage(img)
  }

  const onChangeImageFile = (img) => {
    setImageFile(img)
  }

  const onChangeValue = ({target: { name, value }}) => {
    setErrors({...errors, [name]: '', general: ''})
    setNewProvider({...newProvider, [name]: value})
  }

  const onSubmit = async () => {
    let validateErrors = {}
    if (!newProvider.name) {
      validateErrors['name'] = 'Provider name is required!'
    } else if (newProvider.name.length < 4) {
      validateErrors['name'] = 'Provider name is too short!'
    }
    if (!newProvider.idNumber) {
      validateErrors['idNumber'] = 'ID Number is required!'
    }
    if (Object.values(validateErrors).length) return setErrors(validateErrors)
    setLoading(true)
    if (id) {
      if (imageFile) {
        newProvider.image = imageFile
      }
      let res = await providersApi.edit(id, newProvider)

      if (res.isError) {
        setLoading(false)
        return setErrors({general: res.errorMessage})
      }

      return props.history.push('/providers')
    }

    let response = await providersApi.create({
      ...newProvider,
      status: 'active',
      image: imageFile
    })
    if (response.isError) {
      setLoading(false)
      return setErrors({general: response.errorMessage})
    }

    props.history.push('/providers')
  }

  return (
    <CCard>
      <CCardHeader align="center">
        <h1>{id ? 'Edit Provider' : 'Register Provider'}</h1>
      </CCardHeader>
      <CCardBody>
        <CForm className="form-horizontal" formAction="">
          <div className="row">
            <div className="col-4">
              <UploadImage
                disabled={!permissions.editableProvider || isLoading}
                image={image}
                onUploadImage={onUploadImage}
                onChangeImageFile={onChangeImageFile}
                maxWidth={1024}
                maxHeight={768}
              />
            </div>
            <div className="col-8 row justify-content-center">
              <div className="col-8">
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "120px"}} >Provider Name</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      onChange={onChangeValue}
                      value={newProvider ? newProvider.name : ''}
                      type="text"
                      id="name"
                      name="name"
                      invalid={!!errors["name"]}
                      disabled={!permissions.editableProvider || isLoading}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-user" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors["name"] ? <CInvalidFeedback>{errors["name"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "120px"}} >Full Name</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      onChange={onChangeValue}
                      defaultValue={newProvider ? newProvider.fullName : ''}
                      type="text"
                      id="fullName"
                      name="fullName"
                      disabled={!permissions.editableProvider || isLoading}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-key" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                  </CInputGroup>
                </CFormGroup>
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "120px"}} >ID Number</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      onChange={onChangeValue}
                      value={newProvider ? newProvider.idNumber : ''}
                      type="text"
                      id="idNumber"
                      name="idNumber"
                      invalid={!!errors["idNumber"]}
                      disabled={!permissions.editableProvider || isLoading}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-id-card" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors["idNumber"] ? <CInvalidFeedback>{errors["idNumber"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "120px"}} >Contract Date</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      onChange={onChangeValue}
                      value={newProvider ? newProvider.contractDate : ''}
                      type="date"
                      id="contractDate"
                      name="contractDate"
                      className="no-icon"
                      disabled={!permissions.editableProvider || isLoading}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-file-contract" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                  </CInputGroup>
                </CFormGroup>
                <CFormGroup>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "120px"}} >Address</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      onChange={onChangeValue}
                      value={newProvider ? newProvider.address : ''}
                      type="text"
                      id="text"
                      name="address"
                      disabled={!permissions.editableProvider || isLoading}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText><i className="fas fa-map" style={{ width: '20px' }}/></CInputGroupText>
                    </CInputGroupAppend>
                  </CInputGroup>
                  <CInputGroup className="mb-4">
                      {!!errors["general"] ? <CInput hidden invalid={!!errors["general"]} /> : ""}
                      {!!errors["general"] ? <CInvalidFeedback>{errors["general"]}</CInvalidFeedback> : ""}
                    </CInputGroup>
                </CFormGroup>
              </div>
            </div>
          </div>
          <CCardFooter className="row justify-content-around pt-3 pb-0">
            <div className="col-6 row justify-content-around">
              <CButton disabled={!permissions.editableProvider || isLoading} type="button" className="btn-pill col-4" size="lg" color="primary" onClick={onSubmit}>
                {!!isLoading && <span class="spinner-border spinner-border-sm" role="status" aria-hidden="true"></span>}
                {isLoading ? "Submitting..." : "Submit"}
              </CButton>
              <CButton disabled={isLoading} type="button" className="btn-pill col-4" size="lg" color="dark" onClick={onBack}>Back</CButton>
            </div>
          </CCardFooter>
        </CForm>
      </CCardBody>
    </CCard>
  )
}

const UploadImage = (props) => {
  const [imgPreview, setImgPreview] = useState('')
  const [error, setError] = useState('')

  const chooseImage = (e) => {
    if (!e.target.files || !e.target.files[0]) return
    const img = new Image();
    var objectUrl = URL.createObjectURL(e.target.files[0]);
    img.onload = function () {
      // URL.revokeObjectURL(objectUrl);
      if (this.width > props.maxWidth || this.height > props.maxHeight) {
        return setError(`Image size must less than ${props.maxWidth}*${props.maxHeight}`)
      }
      setError("")
      setImgPreview(objectUrl)
      if (props.onUploadImage) {
        props.onUploadImage(objectUrl)
      }
      if (props.onChangeImageFile) {
        props.onChangeImageFile(e.target.files[0])
      }
    };
    img.src = objectUrl;
  }
  const chooseFileUpload = () => document.getElementById('uploadImage').click()
  return (
    [<div style={{ cursor: 'pointer' }}>
      <input
        hidden
        id="uploadImage"
        type="file"
        accept="image/x-png,image/gif,image/jpeg"
        onChange={chooseImage}
        disabled={props.disabled}
        />
      <img
        src={props.image || imgPreview || 'https://www.seekpng.com/png/detail/110-1100707_person-avatar-placeholder.png'}
        alt="Profile"
        onClick={chooseFileUpload}
        width="100%"
        style={{maxHeight: '250px', maxWidth: '300px'}}
      />
      <div>
        <span style={{
          position: 'relative',
          left: 'calc(50% - 95px)',
          top: '-150px',
          fontSize: '2rem',
          color: '#adadad',
          display: 'block',
        }}
        onClick={chooseFileUpload}
        hidden={!!props.image || !!imgPreview}
        >Profile Image</span>
      </div>
    </div>,
    <CInputGroup className="">
      {!!error ? <CInput hidden invalid={!!error} /> : ""}
      {!!error ? <CInvalidFeedback>{error}</CInvalidFeedback> : ""}
    </CInputGroup>
    ]
  )
}

export default ProviderForm