import React, {useState, useEffect} from 'react'
import {
  CCard,
  CCardBody,
  CCardHeader,
  CButton,
  CFormGroup,
  CInputGroup,
  CInputGroupPrepend,
  CInputGroupAppend,
  CInputGroupText,
  CInput,
  CDataTable,
  CInvalidFeedback
} from '@coreui/react'
import { configsApi } from '../../api'
import Tabs from '../../containers/Tabs'

const Settings = () => {
  const [channels, setChannels] = useState([])
  const [isLoading, setLoading] = useState(false)
  const [errors, setErrors] = useState({})
  const [configs, setConfigs] = useState([])
  const [activeTab, setActiveTab] = useState('configs')
  const tabs = [{
    name: 'configs',
    text: 'Configs'
  }, {
    name: 'channels',
    text: 'Channels'
  }]

  useEffect(() => {
    fetchConfigsAndChannels()
  }, [])

  const fetchConfigsAndChannels = async () => {
    const res = await configsApi.getList()
    if (res.isError) return
    const { channels, configs } = res.data

    setChannels(channels)
    setConfigs(configs)
  }

  const chooseTab = (tabName) => {
    setActiveTab(tabName)
  }

  const changeConfig = ({ target: { name, value }}) => {
    setConfigs(configs.map((c) => {
      if (c.key === name) c.value = value

      return c
    }))
  }

  const changeChannels = ({ target: { name, checked }}) => {
    setChannels(channels.map((c) => {
      if (c.name == name) c.enabled = checked

      return c
    }))
  }

  const saveChannels = async () => {
    setLoading(true)
    const res = await configsApi.update({channels, configs: []})
    setLoading(false)
  }

  const saveConfigs = async () => {
    let errors = {}
    configs.forEach(c => c.value === "" ? errors[c.key] = `${c.key} is required!` : "")
    setLoading(true)
    if (Object.keys(errors).length > 0) return setErrors(errors)
    const res = await configsApi.update({channels: [], configs})
    setLoading(false)
  }

  const saveSettings = async () => {
    activeTab === 'configs' ? saveConfigs() : saveChannels()
  }

  return (
    <CCard>
      <CCardHeader align="center">
        <h1>Settings</h1>
      </CCardHeader>
      <Tabs
        className="col-12"
        tabs={tabs}
        chooseTab={chooseTab}
      />
      <CCardBody>
        <div className="row justify-content-center">
          {activeTab === 'configs' &&
            <div className="row col-8 justify-content-center">
              {configs.map((c, idx) => (
                <CFormGroup className="col-8 row" key={idx}>
                  <CInputGroup>
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "200px"}} >{c.key}</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      onChange={changeConfig}
                      defaultValue={c.value || ''}
                      type="text"
                      id={`${c.key}_${idx}`}
                      name={c.key}
                      invalid={!!errors[c.key]}
                      disabled={isLoading}
                    />
                    <CInputGroupAppend>
                      <CInputGroupText style={{ width: '60px' }}>{c.unit}</CInputGroupText>
                    </CInputGroupAppend>
                    {!!errors[c.key] ? <CInvalidFeedback>{errors[c.key]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                </CFormGroup>
              ))}
            </div>
          }
          {activeTab === 'channels' &&
            <div className="col-8">
              <CDataTable
                items={channels}
                fields={['channel', 'enabled']}
                hover
                // onRowClick={onSelectRow}
                // clickableRows
                scopedSlots = {{
                  'channel':
                    (item)=>(
                      <td style={{verticalAlign: "middle"}}>
                        {item.name}
                      </td>
                    ),
                  'enabled':
                    (item)=>(
                      <td className="pl-4">
                        <input
                          className="cursor-pointer"
                          style={{width: "20px", height: "20px"}}
                          type="checkbox"
                          checked={item.enabled}
                          onChange={changeChannels}
                          name={item.name}
                        />
                      </td>
                    ),
                }}
              />
            </div>
          }
        </div>
        <div className="row justify-content-center">
          <div className="col-3">
            <CButton
              disabled={false}
              type="button"
              className="btn-pill col-12"
              size="lg"
              color="primary"
              onClick={saveSettings}
            >
              {!!isLoading && <span class="spinner-border spinner-border-sm" role="status" aria-hidden="true"></span>}
              {isLoading ? "Saving..." : "Save"}
            </CButton>
          </div>
        </div>
      </CCardBody>
    </CCard>
  )
}

export default Settings
