import React, { useEffect, useState } from 'react'
import {
  CButton,
  CCard,
  CCardBody,
  CCol,
  CContainer,
  CForm,
  CInput,
  CInputGroup,
  CInputGroupPrepend,
  CInputGroupText,
  CInvalidFeedback,
  CSelect,
  CRow,
} from '@coreui/react'
import { authApi, providersApi, usersApi } from '../../api'
import { MODELS, ACTIONS } from '../../constants'

const Register = (props) => {
  const { history, match: {params: { id }} } = props
  const [subRoles, setSubRoles] = useState([])
  const [user, setUser] = useState(null)
  const [providers, setProviders] = useState([])
  const [errors, setErrors] = useState({common: ''})
  const [newUser, setNewUser] = useState({})
  const [isLoading, setLoading] = useState(false)
  const [permissions, setPermissions] = useState({
    editableProvider: false,
    editableUser: false,
  })
  useEffect(() => {
    const fetchPermissions = async () => {
      const editableProvider = await authApi.checkPermission(MODELS.PROVIDER_DETAILS, ACTIONS.EDIT)
      const editableUser = await authApi.checkPermission(MODELS.USER_DETAILS, ACTIONS.EDIT)

      setPermissions({ editableUser, editableProvider })
    }

    fetchPermissions()
  }, [])
  useEffect(() => {
    const fetchUser = async (id) => {
      setLoading(true)
      let res = await usersApi.getOne(id)
      setLoading(false)
      if (res.isError) {
        return setErrors({...errors, common: res.errorMessage})
      }

      setNewUser(res.data)
    }

    if (id) fetchUser(id)
  }, [id])

  useEffect(() => {
    if (user) {
      let roles = authApi.getSubRoles(user.role)
      setSubRoles(roles)
      if (!id) {
        setNewUser({...newUser, providerId: user.providerId})
      }
    }
  }, [user])

  useEffect(() => {
    const fetchProviders = async () => {
      let res = await providersApi.getListForSelect()
      if (!res.isError) {
        setProviders(res.data.providers)
      }
    }
    fetchProviders()
  }, [])

  useEffect(() => {
    const fetchUser = async () => {
      let res = await authApi.checkAuth()
      if (res.isError) {
        return setErrors({...errors, common: res.errorMessage})
      }
      setUser(res.data)
    }

    fetchUser()
  }, [])
  const onChangeValue = ({ target: { name, value }}) => {
    setNewUser({...newUser, [name]: value})
  }

  const validateForm = () => {
    let { name, role, userName, password, rePassword } = newUser
    let newError = {}
    if (!name) {
      newError.name = 'Full Name is required!'
    }
    if (!userName) {
      newError.userName = 'User Name is required!'
    }
    if (!id && !password) {
      newError.password = 'Password is required!'
    }
    if (rePassword !== password) {
      newError.rePassword = 'rePassword is not correct'
    }
    if (!role) {
      newError.role = 'Please select role!'
    }

    return newError
  }

  const onSubmit = async () => {
    let error = validateForm()
    if (error && Object.keys(error).length) {
      return setErrors(error)
    }
    setLoading(true)
    if (id) {
      if (!newUser.password) {
        delete newUser.password
        delete newUser.rePassword
      }
      let res = await usersApi.edit(id, newUser)
      setLoading(false)
      if (!res.isError) {
        return history.push('/users')
      }
      return setErrors({common: res.errorMessage})
    }

    let res = await usersApi.create(newUser)
    setLoading(false)
    if (!res.isError) {
      return history.push('/users')
    }
    setErrors({common: res.errorMessage})
  }

  return (
    <div className="c-app c-default-layout flex-row align-items-center">
      <CContainer>
        <CRow className="justify-content-center">
          <CCol md="9" lg="7" xl="6">
            <CCard className="mx-4">
              <CCardBody className="p-4">
                <CForm>
                  <h1>{id ? "User" : "Register"}</h1>
                  <p className="text-muted">
                  {id
                    ? "Edit User"
                    : "Create new user"
                  }</p>
                  <CInputGroup className="mb-3">
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "110px"}} >
                        Full Name
                      </CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      invalid={!!errors["name"]}
                      type="text"
                      name="name"
                      value={newUser.name || ""}
                      disabled={!permissions.editableUser || isLoading}
                      onChange={onChangeValue}
                      placeholder="Full Name"
                      autoComplete="name"
                    />
                    {!!errors["name"] ? <CInvalidFeedback>{errors["name"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                  <CInputGroup className="mb-3">
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "110px"}} >
                        Username
                      </CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      invalid={!!errors["userName"]}
                      type="text"
                      name="userName"
                      value={newUser.userName || ""}
                      disabled={!permissions.editableUser || isLoading}
                      onChange={onChangeValue}
                      placeholder="username"
                      autoComplete="username"
                    />
                    {!!errors["userName"] ? <CInvalidFeedback>{errors["userName"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                  <CInputGroup className="mb-3">
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "110px"}} >Email</CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      invalid={!!errors["email"]}
                      type="text"
                      name="email"
                      value={newUser.email || ""}
                      disabled={!permissions.editableUser || isLoading}
                      onChange={onChangeValue}
                      placeholder="Email" autoComplete="email" />
                      {!!errors["email"] ? <CInvalidFeedback>{errors["email"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                  <CInputGroup className="mb-3">
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "110px"}} >Role</CInputGroupText>
                    </CInputGroupPrepend>
                    <CSelect
                      onChange={onChangeValue}
                      value={newUser.role || ''}
                      disabled={!permissions.editableUser || isLoading}
                      type="text"
                      id="role"
                      name="role"
                      invalid={!!errors["role"]}
                    >
                      <option value="">Select Role</option>
                      {subRoles.map((role) => (
                        <option value={role.value}>{role.name}</option>
                      ))}
                    </CSelect>
                    {!!errors["role"] ? <CInvalidFeedback>{errors["role"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                  <CInputGroup className="mb-3">
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "110px"}} >Provider</CInputGroupText>
                    </CInputGroupPrepend>
                      <CSelect
                        onChange={onChangeValue}
                        value={newUser.providerId || ''}
                        disabled={!permissions.editableUser || !permissions.editableProvider || isLoading}
                        type="text"
                        id="providerId"
                        name="providerId"
                        invalid={!!errors["providerId"]}
                      >
                        <option value="">Select Provider</option>
                        {providers.map((provider) => (
                          <option value={provider.id}>{provider.name}</option>
                        ))}
                      </CSelect>
                    {!!errors["providerId"] ? <CInvalidFeedback>{errors["providerId"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                  <CInputGroup className="mb-3">
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "110px"}} >
                        Password
                      </CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      invalid={!!errors["password"]}
                      type="password"
                      name="password"
                      value={newUser.password || ""}
                      disabled={!permissions.editableUser || isLoading}
                      onChange={onChangeValue}
                      placeholder="Password"
                      autoComplete="new-password"
                    />
                    {!!errors["password"] ? <CInvalidFeedback>{errors["password"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                  <CInputGroup className="mb-4">
                    <CInputGroupPrepend>
                      <CInputGroupText style={{ width: "110px"}} >
                        Re-Password
                      </CInputGroupText>
                    </CInputGroupPrepend>
                    <CInput
                      invalid={!!errors["rePassword"]}
                      type="password"
                      name="rePassword"
                      value={newUser.rePassword || ""}
                      disabled={!permissions.editableUser || isLoading}
                      onChange={onChangeValue}
                      placeholder="Repeat password" autoComplete="new-password"
                    />
                    {!!errors["rePassword"] ? <CInvalidFeedback>{errors["rePassword"]}</CInvalidFeedback> : ""}
                  </CInputGroup>
                    <CInputGroup className="mb-4">
                      {!!errors["common"] ? <CInput hidden invalid={!!errors["common"]} /> : ""}
                      {!!errors["common"] ? <CInvalidFeedback>{errors["common"]}</CInvalidFeedback> : ""}
                    </CInputGroup>
                  <CButton type="button" disabled={!permissions.editableUser || isLoading} color="success" block onClick={onSubmit}>
                    {!!isLoading && <span class="spinner-border spinner-border-sm" role="status" aria-hidden="true"></span>}
                    {isLoading ? "Submitting..." : id ? "Save" : "Create Account"}
                  </CButton>
                </CForm>
              </CCardBody>
            </CCard>
          </CCol>
        </CRow>
      </CContainer>
    </div>
  )
}

export default Register
