import React, { useState, useEffect } from 'react'
import { useHistory, useLocation } from 'react-router-dom'
import {
  CBadge,
  CCard,
  CCardBody,
  CCardHeader,
  CCol,
  CDataTable,
  CRow,
  CPagination,
  CButton
} from '@coreui/react'
import { usersApi, authApi, providersApi } from '../../api'
import Search from '../../containers/Search'

const getBadge = status => {
  switch (status) {
    case 'active': return 'success'
    case 'inactive': return 'secondary'
    case 'pending': return 'warning'
    case 'banned': return 'danger'
    default: return 'primary'
  }
}

const Users = (props) => {
  const history = useHistory()
  const [users, setUsers] = useState([])
  const [subRoles, setSubRoles] = useState([])
  const queryPage = useLocation().search.match(/page=([0-9]+)/, '')
  const [count, setCount] = useState(10)
  const [sugProviders, setSugProviders] = useState([])
  const limit = 10
  const [filter, setFilter] = useState({})
  const currentPage = Number(queryPage && queryPage[1] ? queryPage[1] : 1)
  const [page, setPage] = useState(currentPage || 1)

  useEffect(() => {
    const fetchSugProviders = async () => {
      let res = await providersApi.getListForSelect()
      if (!res.isError) {
        setSugProviders(res.data.providers.map(p => p.name))
      }
    }
    fetchUsers({}, currentPage, true)
    fetchSugProviders()
  }, [])

  useEffect(() => {
    const fetchUserAndSubRoles = async () => {
      let res = await authApi.fetchUserInfo()
      if (!res.isError) {
        let roles = authApi.getSubRoles(res.data.role)
        setSubRoles(roles)
      }
    }
    fetchUserAndSubRoles()
  }, [])

  const fetchUsers = async (filter, page, fetchCount) => {
    let res = await usersApi.getList({...filter, page, limit, fetchCount})
    if (!res.isError) {
      setPage(page)
      setUsers(res.data.users)
      if (fetchCount) {
        setCount(res.data.count)
      }
    }
  }

  const pageChange = newPage => {
    if ((newPage || 1) === page) return
    fetchUsers(filter, newPage, false)
  }

  const onRegister = () => {
    history.push('/users/new')
  }

  useEffect(() => {
    currentPage !== page && setPage(currentPage)
  }, [currentPage, page])

  const onSearch = (fieldName, value) => {
    let newFilter = {...filter, [fieldName]: value}
    setFilter(newFilter)

    fetchUsers(newFilter, 1, true)
  }

  return (
    <CRow>
      <CCol xl={12}>
        <CCard>
          <CCardHeader align="center">
            <h1 style={{ display: 'inline-block' }}>User List</h1>
            <CButton
              onClick={onRegister}
              color="success"
              style={{ position: "relative", top: '-5px', left: 'calc(50% - 152px)' }}
            >
              Create User
            </CButton>
          </CCardHeader>
          <CCardBody>
            <Search
              items={[{
                name: "status",
                type: "select",
                placeholder: "Status",
                options: [{
                  value: 'active',
                  text: 'Active',
                },{
                  value: 'inactive',
                  text: 'Inactive',
                }],
                gridSize: 2
              }, {
                name: "role",
                type: "select",
                placeholder: "Role",
                options: subRoles.map((r) => ({
                  value: r.value,
                  text: r.value
                })),
                gridSize: 2
              }, {
                name: "name",
                type: "text",
                placeholder: "Name...",
                gridSize: 3,
              }, {
                name: "providerName",
                type: "text",
                placeholder: "Provider Name...",
                gridSize: 3,
                autocomplete: true,
                suggestions: sugProviders,
              }]}
              search={onSearch}
            />
            <CDataTable
              items={users}
              fields={[
                { key: 'name', _classes: 'font-weight-bold' },
                'role', 'providerName', 'status'
              ]}
              hover
              striped
              itemsPerPage={limit}
              activePage={page}
              clickableRows
              onRowClick={(item) => history.push(`/users/${item.id}`)}
              scopedSlots = {{
                'status':
                  (item)=>(
                    <td>
                      <CBadge color={getBadge(item.status)}>
                        {item.status}
                      </CBadge>
                    </td>
                  ),
                'providerName':
                  (item)=>(
                    <td>
                      {item.providerName || ''}
                    </td>
                  )
              }}
            />
            <CPagination
              activePage={page}
              onActivePageChange={pageChange}
              pages={Math.ceil(count / limit)}
              doubleArrows={false}
              align="center"
            />
          </CCardBody>
        </CCard>
      </CCol>
    </CRow>
  )
}

export default Users
